SystemOrganization addCategory: #'Ring-Core-Kernel'!

!Metaclass methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:46'!
asFullRingDefinition

	^ self theNonMetaClass asFullRingDefinition theMetaClass! !

!Metaclass methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:46'!
asRingDefinition

	^ self theNonMetaClass asRingDefinition theMetaClass! !

!ClassTrait methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:45'!
asFullRingDefinition

	^ self theNonMetaClass asFullRingDefinition theMetaClass! !

!ClassTrait methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:45'!
asRingDefinition

	^ self theNonMetaClass asRingDefinition theMetaClass! !

!Behavior methodsFor: '*Ring-Core-Kernel' stamp: ''!
methodNamed: aSelector

	^ self methodDict at: aSelector! !

!Behavior methodsFor: '*Ring-Core-Kernel' stamp: ''!
methods
	
	^ self methodDict values! !

!Behavior methodsFor: '*Ring-Core-Kernel' stamp: ''!
methodsInProtocol: aString

	^ (self organization listAtCategoryNamed: aString) 
			collect: [:each | (self methodDict at: each) ]! !

!Behavior methodsFor: '*Ring-Core-Kernel' stamp: ''!
protocols
	
	^ self organization categories copy! !

!Class methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:45'!
asFullRingDefinition
	"A behavior is converted to a ring class including its variables, methods, direct superclass, direct subclasses and the package in which is loaded.
	Active methods are generated and each knows its package as well.
	Note that for its direct superclass and subclasses no full definitions are requested. If you need to traverse hierarchies use #asRingDefinitionWithMethods:withSuperclasses:withSubclasses:withPackages:"
	
	| rgClass rgSuper rgSub rgMethod packageKeys |
	rgClass:= self asRingDefinition.
	rgClass package: (RGContainer packageOfClass: rgClass).
	
	self superclass notNil ifTrue: [ 
			rgSuper := self superclass asRingDefinition.
			rgClass superclass: rgSuper ].

	self subclasses do:[ :each |
		rgSub := each asRingDefinition.
		rgSub superclass: rgClass ].
	
	packageKeys := RGContainer packageKeys.
	self methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			rgClass addMethod: rgMethod.
			rgMethod package: (RGContainer packageOfMethod: rgMethod using: packageKeys) ].
	self theMetaClass methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			rgClass theMetaClass addMethod: rgMethod.
			rgMethod package: (RGContainer packageOfMethod: rgMethod using: packageKeys) ].
	
	^ rgClass
! !

!Class methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:45'!
asRingDefinition
	"A behavior is converted to a ring class. Only the receiver and its variables (instance, class, pools) are converted.
	Methods, superclasses, subclasses are not generated"
	
	| ring |
	ring := (RGFactory current createClassNamed: self name)
		category: self category;
		superclassName: self superclass name;  
		traitCompositionSource: self traitCompositionString;
		addInstanceVariables: self instVarNames;
		addClassVariables: self classVarNames;
		addPoolDictionaries: self poolDictionaryNames;
		comment: self organization classComment;
		stamp: self organization commentStamp;
		definitionSource: self definition;
		withMetaclass.
	ring theMetaClass 
		traitCompositionSource: self theMetaClass traitCompositionString;
		definitionSource: self theMetaClass definition;
		addInstanceVariables: self theMetaClass instVarNames.  
	^ ring! !

!Class methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 15:32'!
asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: packageKeys in: aRGSlice
	
	| rgClass rgMethod rgSuper rgSub subs |	
	rgClass := self asRingDefinition.
	aRGSlice loadClass: rgClass using: packageKeys.

	methodsBoolean ifTrue: [
		self methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			aRGSlice loadMethod: rgMethod inClass: rgClass using: packageKeys ].
		self theMetaClass methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			aRGSlice loadMethod: rgMethod inClass: rgClass theMetaClass using: packageKeys ] ].

	supersBoolean ifTrue: [
		self superclass isNil
			ifFalse:[
				rgSuper := aRGSlice classNamed: self superclass name.
				rgSuper isNil ifTrue: [
				 	rgSuper := self superclass asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: packageKeys in: aRGSlice ].
				rgClass superclass: rgSuper ] ].
	
	subsBoolean ifTrue: [
		subs := self subclasses select:[ :sub| sub isMeta not ].
		rgClass name = #Trait ifTrue: [ subs := aRGSlice environment allTraits ].
		subs do:[ :each|
			rgSub := aRGSlice classNamed: each name.
			rgSub isNil ifTrue: [
				rgSub := each asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: packageKeys in: aRGSlice ].
			rgSub superclass: rgClass ] ].
	
	^rgClass! !

!Class methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 15:35'!
asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackages: packsBoolean
	"Retrieves a ring class based on the receiver.
	The data loaded in the class (active methods, superclasses, subclasses and packages) is requested by the users.
	As it may need to traverse hierarchies for retrieving super and subclasses a ring slice is created as the container for every class, method and package.
	To retrieve the slice:  aRGClass environment "
	| rgClass rgSlice rgPackage rgPackageKeys |
	
	rgSlice := RGSlice named: #fromImage.
	packsBoolean ifTrue: [ 
		rgPackageKeys := rgSlice loadPackagesFromImage ].
	rgClass := self asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: rgPackageKeys in: rgSlice.
	rgSlice cleanEmptyPackages.
	rgSlice loadTraitUsers.
		
	^ rgClass! !

!ChangeRecord methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 7/8/2011 15:31'!
asRingDefinition
	"Retrieves a RGMethodDefinition object based on the data of the receiver"
	
	self type == #method
		ifTrue: [ ^self createMethodDefinition ].
	self type == #classComment
		ifTrue: [ ^self createCommentDefinition ].
	^nil! !

!ChangeRecord methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 5/12/2011 14:32'!
createCommentDefinition
	"Retrieves a RGCommentDefinition object based on the data of the receiver"
	
	^(RGFactory current createCommentDefinition)
		parentName: self commentClass name;
		content: self string;
		stamp: stamp;
		yourself
	! !

!ChangeRecord methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 8/31/2011 14:09'!
createMethodDefinition
	"Retrieves a RGMethodDefinition object based on the data of the receiver"
	
	^(RGFactory current createMethodNamed: self methodSelector)
		parentName: self methodClass name;
		isMetaSide: meta;  
		sourceCode: self string;
		protocol: category;
		stamp: stamp;
		yourself! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:08'!
asActiveRingDefinition

	"Retrieves an active RGMethodDefinition object based on the data of the receiver.
	Source, protocol and stamp are retrieved from the compiled method"
	| ring |
	ring := (RGFactory current createMethodNamed: self selector)
				parentName: self methodClass name;
				isMetaSide: self methodClass isMeta;
				asActive.
	^ ring! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:30'!
asFullRingDefinition

	"Retrieves an active RGMethodDefinition which knows its parent <class>.
	Note that the full conversion does not happen at the level of the class. If you need that request asFullRingDefinition to the class"
	| rgClass rgMethod |
	rgClass := self realClass asRingDefinition.
	rgMethod := self asActiveRingDefinition.
	rgMethod package: (RGContainer packageOfMethod: rgMethod).
	rgClass addMethod: rgMethod.
	^ rgMethod! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'MarcusDenker 4/29/2012 10:33'!
asHistoricalRingDefinition

	"Retrieves a historical RGMethodDefinition object based on the data of the receiver.
	Source, protocol and stamp are retrieved from the source file method"
	| ring |
	ring := (RGFactory current createMethodNamed: self selector)
				parentName: self methodClass name;
				isMetaSide: self methodClass isMeta.

	self sourcePointer isZero
		ifTrue: [ "this should not happen but sometimes the system looks corrupted"
			ring protocol: self category;
				sourceCode: self sourceCode;
				stamp: self timeStamp ]
		ifFalse: [ 
			ring sourcePointer: self sourcePointer ].
	ring asHistorical.	
	
	^ ring! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'MarcusDenker 4/29/2012 10:33'!
asPassiveRingDefinition

	"Retrieves a passive RGMethodDefinition object based on the data of the receiver.
	Source, protocol and stamp are retrieved from value assigned in creation"
	| ring |
	ring := (RGFactory current createMethodNamed: self selector)
				parentName: self methodClass name;
				isMetaSide: self methodClass isMeta;
				protocol: self category;
				sourceCode: self sourceCode;
				stamp: self timeStamp;
				asPassive.
	^ ring! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:09'!
asRingDefinition
	"Retrieves an active RGMethodDefinition object based on the receiver.
	Note that its class is not converted."
	
	^ self asActiveRingDefinition! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'CamilloBruni 5/21/2012 15:29'!
getSourceFromFile: file
	"Read the source code from file, determining source file index and
	file position from the last 3 bytes of this method."
	| position |
	(position := self filePosition) = 0 ifTrue: [^ nil].
	"assime this is a valid file"
	file position: position.
	^ file nextChunk! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/21/2011 17:49'!
methodReference

	| class selector |
	class := self methodClass ifNil: [^nil].
	selector := self selector ifNil: [^nil].
	^RGMethodDefinition realClass: class selector: selector.
	! !

!CompiledMethod methodsFor: '*Ring-Core-Kernel' stamp: 'MarianoMartinezPeck 6/26/2012 10:41'!
realClass
	"answer the class that I am installed in"
	
	^ self methodClass! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/1/2011 19:34'!
getPreambleFrom: aFileStream at: position

	|  writeStream |
	writeStream := String new writeStream.
	position to: 0 by: -1 do: [ :p | 
			| c | 
			aFileStream position: p.
			c := aFileStream basicNext.
			c == $!!
				ifTrue: [^ writeStream contents reversed]
				ifFalse: [writeStream nextPut: c]]! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 8/2/2011 11:12'!
protocolAt: sourcePointer
	"Answer the protocol for a given method, retrieved from the sources or changes file. Answer unfound protocol if no protocol is available"
				
	^ self protocolAt: sourcePointer for: self sourceDataPointers! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/1/2011 00:09'!
protocolAt: sourcePointer for: sourceDataPointers
	"Answer the protocol for a given method, retrieved from the sources or changes file. Answer unfound protocol if no protocol is available."	
				
	| preamble protocol tokens protocolPosition |
	protocol := 'unfound protocol'.
	"this is to indicate that the tagging in the source does not use the correct format.
	We will have to fix that. For example some traits methods are wrongly tagged.
	see http://code.google.com/p/pharo/issues/detail?id=4581"
	preamble := self sourcedDataAt: sourcePointer.	
	preamble = 'Trait method' ifTrue: [ ^nil ].
	(preamble findString: (sourceDataPointers key) startingAt: 1) > 0
			ifTrue: [ tokens := Scanner new scanTokens: preamble.
				     	protocolPosition := tokens indexOf: sourceDataPointers key.
					protocolPosition = 0
						ifFalse: [
							"New format gives change protocol and unified prior pointer"
							protocol := tokens at: protocolPosition + 1] ].
 	^protocol! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/1/2011 19:34'!
sourceCodeAt: sourcePointer 
	"Retrieves the source code for the pointer received as argument"
	
	| fileIndex filePosition |
	filePosition := self filePositionFromSourcePointer: sourcePointer.
	^filePosition = 0
		ifTrue: [ '' ]
		ifFalse:[ fileIndex := self fileIndexFromSourcePointer: sourcePointer.
				(RemoteString newFileNumber: fileIndex position: filePosition) string ]! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/1/2011 21:45'!
sourceDataPointers
	"Retrieves the combination key to look for information of the receiver in the source file"
	"'Point methodsFor: ''point functions'' stamp: ''lr 7/4/2009 10:42'''."
	
	^'methodsFor:' -> #stamp:! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/1/2011 19:34'!
sourcedDataAt: sourcePointer

	| index position file preamble data tokens dataPosition |
	index:= self fileIndexFromSourcePointer: sourcePointer.
	index = 0 ifTrue: [ ^'' ].  "no source pointer for this element"
	position := self filePositionFromSourcePointer: sourcePointer.
	file := self at: index.
	file ifNil: [ ^'' ].  "sources file not available"
	"file does not exist happens in secure mode"
	file := [file readOnlyCopy] 
			on: FileDoesNotExistException 
			do:[:ex| ^'' ].
	self flag: 'I do not know why 3 is a good number'.
	preamble :=  self getPreambleFrom: file at: (0 max: position - 3).
	file close.
	^preamble! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/1/2011 19:37'!
timeStampAt: sourcePointer 
	"Answer the authoring time-stamp for the given method, retrieved from the sources or changes file. Answer the empty string if no time stamp is available."		
				
	^ self timeStampAt: sourcePointer for: self  sourceDataPointers ! !

!SourceFileArray methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 8/1/2011 21:48'!
timeStampAt: sourcePointer for: sourceDataPointers
	"Answer the authoring time-stamp for the given method, retrieved from the sources or changes file. Answer the empty string if no time stamp is available."		
				
	| preamble stamp tokens stampPosition |
	stamp := ''.
	preamble := self sourcedDataAt: sourcePointer.
	(preamble findString: (sourceDataPointers key) startingAt: 1) > 0
			ifTrue: [ tokens := Scanner new scanTokens: preamble.
				     	stampPosition := tokens indexOf: sourceDataPointers value.
					stampPosition = 0
						ifFalse: [
							"New format gives change stamp and unified prior pointer"
							stamp := tokens at: stampPosition + 1]].
 	^stamp! !

!TPureBehavior methodsFor: '*Ring-Core-Kernel' stamp: 'CamilloBruni 10/17/2012 10:05'!
methodNamed: aSelector

	^ self methodDict at: aSelector! !

!TPureBehavior methodsFor: '*Ring-Core-Kernel' stamp: 'CamilloBruni 10/17/2012 10:05'!
methods
	
	^ self methodDict values! !

!TPureBehavior methodsFor: '*Ring-Core-Kernel' stamp: 'CamilloBruni 10/17/2012 10:05'!
methodsInProtocol: aString

	^ (self organization listAtCategoryNamed: aString) 
			collect: [:each | (self methodDict at: each) ]! !

!TPureBehavior methodsFor: '*Ring-Core-Kernel' stamp: 'CamilloBruni 10/17/2012 10:04'!
protocols
	
	^ self organization categories copy! !

!Trait methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 16:42'!
asFullRingDefinition

	"A trait is converted to a ring trait including its methods, direct superclass and the package in which is loaded.
	Active methods are generated and each knows its package as well.
	Note that for its direct superclass no full definitions is requested. If you need to traverse hierarchies use #asRingDefinitionWithMethods:withSuperclasses:withSubclasses:withPackages:"	
	| rgClass rgSuper rgSub rgMethod packageKeys |
	rgClass:= self asRingDefinition.
	rgClass package: (RGContainer packageOfClass: rgClass).
	
	rgSuper := Trait asRingDefinition.
	rgClass superclass: rgSuper.

	packageKeys := RGContainer packageKeys.
	self methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			rgClass addMethod: rgMethod.
			rgMethod package: (RGContainer packageOfMethod: rgMethod using: packageKeys) ].
	self theMetaClass methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			rgClass theMetaClass addMethod: rgMethod.
			rgMethod package: (RGContainer packageOfMethod: rgMethod using: packageKeys) ].
	
	^rgClass
! !

!Trait methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 13:06'!
asRingDefinition
	"A triat is converted to a ring class. Only the receiver is converted.
	Methods, superclasses, subclasses are not generated"

	| ring |
	ring:= (RGFactory current createTraitNamed: self name)
		category: self category;
		superclassName: #Trait;  
		traitCompositionSource: self traitCompositionString;
		comment: self organization classComment;
		stamp: self organization commentStamp;
		definitionSource: self definition;
		withMetaclass.
		
	ring theMetaClass 
		traitCompositionSource: self theMetaClass traitCompositionString;
		definitionSource: self theMetaClass definition.
	^ring! !

!Trait methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 15:33'!
asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: packageKeys in: aRGSlice
	
	| rgClass rgMethod rgSuper rgSub sup |	
	rgClass := self asRingDefinition.
	aRGSlice loadClass: rgClass using: packageKeys.

	methodsBoolean ifTrue: [
		self methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			aRGSlice loadMethod: rgMethod inClass: rgClass using: packageKeys ].
		self theMetaClass methodsDo:[ :mth|  
			rgMethod := mth asActiveRingDefinition.
			aRGSlice loadMethod: rgMethod inClass: rgClass theMetaClass using: packageKeys ] ].

	supersBoolean ifTrue: [
		rgSuper := aRGSlice classNamed: rgClass superclassName.
		rgSuper isNil ifTrue: [
			sup := aRGSlice environment classNamed: rgClass superclassName.
			rgSuper := sup asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: packageKeys in: aRGSlice ].
		rgClass superclass: rgSuper ].
	
	^rgClass! !

!Trait methodsFor: '*Ring-Core-Kernel' stamp: 'VeronicaUquillas 9/19/2011 15:35'!
asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackages: packsBoolean

	"Retrieves a ring trait based on the receiver.
	The data loaded in the trait (active methods, superclasses, subclasses and packages) is requested by the users.
	As it may need to traverse hierarchies for retrieving super and sub classes a ring slice is created as the container for every class, method and package.
	To retrieve the slice:  aRGClass environment "
	| rgClass rgSlice rgPackage rgPackageKeys |
	
	rgSlice := RGSlice named: #fromImage.
	packsBoolean ifTrue: [ 
		rgPackageKeys := rgSlice loadPackagesFromImage ].
	rgClass := self asRingDefinitionWithMethods: methodsBoolean withSuperclasses: supersBoolean withSubclasses: subsBoolean withPackageKeys: rgPackageKeys in: rgSlice.
	rgSlice cleanEmptyPackages.
	rgSlice loadTraitUsers.
	
	^ rgClass! !

!Object methodsFor: '*Ring-Core-Kernel' stamp: 'StephaneDucasse 7/16/2011 22:53'!
isRingObject

	^false! !

Object subclass: #RGFactory
	instanceVariableNames: ''
	classVariableNames: 'CurrentFactories'
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGFactory commentStamp: 'VeronicaUquillas 5/6/2011 12:39' prior: 0!
RGFactory is responsible for creating instances of the Ring meta-model classes!

!RGFactory class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 5/11/2011 13:26'!
current

	^CurrentFactories 
		at: self species
		ifAbsentPut:[ self new ]
! !

!RGFactory class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 5/11/2011 13:25'!
initialize
	"RGFactory initialize"
	CurrentFactories:= IdentityDictionary new.! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:04'!
classClass

	^RGClassDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:06'!
classInstanceVariableClass

	^RGClassInstanceVariableDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:05'!
classVariableClass

	^RGClassVariableDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:06'!
commentClass

	^RGCommentDefinition! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:07'!
createClassInstanceVariableNamed: aSymbol

	^self createInstanceOfClass: self classInstanceVariableClass named: aSymbol! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createClassNamed: aSymbol

	^self createInstanceOfClass: self classClass named: aSymbol! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createClassNamed: aSymbol subclassOf: aRGClassDefinition

	^(self createClassNamed: aSymbol) superclass: aRGClassDefinition! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createClassVariableNamed: aSymbol

	^self createInstanceOfClass: self classVariableClass named: aSymbol! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createClassVariableNamed: aSymbol parent: aRGClassDefinition

	^(self createClassVariableNamed: aSymbol) parent: aRGClassDefinition! !

!RGFactory methodsFor: 'factory' stamp: 'VeronicaUquillas 5/17/2011 14:48'!
createComment

	^self commentClass new! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:09'!
createComment: content parent: aRGObject
	"Creates a RGCommentDefinition object if content is not a RGCommentDefinition object already
	and if is not empty"

	content isEmptyOrNil
		ifTrue: [ ^nil ].

	^content isRingObject not
		ifTrue: [
			self createComment
				parent: aRGObject;
				content: content; 
				yourself ]
		ifFalse: [ content ]! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createGlobalVariableNamed: aSymbol

	^self createInstanceOfClass: self globalVariableClass named: aSymbol! !

!RGFactory methodsFor: 'private' stamp: 'StephaneDucasse 7/19/2011 10:15'!
createInstanceOfClass: aClass named: aSymbol
	
	^aClass named: aSymbol asSymbol! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createInstanceVariableNamed: aSymbol

	^self createInstanceOfClass: self instanceVariableClass named: aSymbol! !

!RGFactory methodsFor: 'factory-variable' stamp: 'VeronicaUquillas 8/16/2011 15:23'!
createInstanceVariableNamed: aSymbol parent: aRGClassDescriptionDefinition
	^(self 
		createInstanceOfClass: 
			(aRGClassDescriptionDefinition isMeta 
				ifTrue: [ self classInstanceVariableClass ]
				ifFalse:[ self instanceVariableClass ])
		named: aSymbol) parent: aRGClassDescriptionDefinition! !

!RGFactory methodsFor: 'factory' stamp: 'VeronicaUquillas 4/28/2011 15:14'!
createMetaclassOf: aRGClassDefinition

	^self metaclassClass class: aRGClassDefinition! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:09'!
createMetatraitNamed: aSymbol

	^self createInstanceOfClass: self metatraitClass named: aSymbol! !

!RGFactory methodsFor: 'factory' stamp: 'VeronicaUquillas 4/28/2011 15:36'!
createMetatraitOf: aRGClassDefinition

	^self metatraitClass class: aRGClassDefinition! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:09'!
createMethodNamed: aSymbol

	^self createInstanceOfClass: self methodClass named: aSymbol! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:14'!
createMethodNamed: aSymbol parent: aRGBehaviorDefinition
	"Create a method whose selector is aSymbol whose class is a ClassDefinition. 
	Other creation methods may use instead a Smalltalk class."
	
	^(self createMethodNamed: aSymbol) parent: aRGBehaviorDefinition! !

!RGFactory methodsFor: 'factory' stamp: 'VeronicaUquillas 9/18/2011 15:15'!
createMethodNamed: aSymbol realParent: aClass
	"Create a method whose selector is aSymbol whose class is the Smalltalk class aClass. Other creation methods may use ClassDefinition instead of Smalltalk class."

	^(aClass >> aSymbol asSymbol) asActiveRingDefinition ! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createPoolVariableNamed: aSymbol

	^self createInstanceOfClass: self poolVariableClass named: aSymbol! !

!RGFactory methodsFor: 'factory-variable' stamp: 'StephaneDucasse 7/19/2011 10:08'!
createPoolVariableNamed: aSymbol parent: aRGClassDefinition

	^(self createPoolVariableNamed: aSymbol) parent: aRGClassDefinition! !

!RGFactory methodsFor: 'factory-variable' stamp: 'VeronicaUquillas 8/2/2011 14:00'!
createSharedPoolNamed: aSymbol
	"A shared pool is a class inheriting from #SharedPool"

	^(self createInstanceOfClass: self classClass named: aSymbol)
			superclassName: #SharedPool;
			isPool: true;
			yourself! !

!RGFactory methodsFor: 'factory' stamp: 'StephaneDucasse 7/19/2011 10:15'!
createTraitNamed: aSymbol

	^self createInstanceOfClass: self traitClass named: aSymbol! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/28/2011 16:16'!
globalVariableClass

	^RGGlobalVariableDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:05'!
instanceVariableClass

	^RGInstanceVariableDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:05'!
metaclassClass

	^RGMetaclassDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:07'!
metatraitClass

	^RGMetatraitDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:05'!
methodClass

	^RGMethodDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:06'!
poolVariableClass

	^RGPoolVariableDefinition! !

!RGFactory methodsFor: 'class factory' stamp: 'VeronicaUquillas 4/27/2011 22:07'!
traitClass

	^RGTraitDefinition! !

Object subclass: #RGObject
	instanceVariableNames: 'annotations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGObject commentStamp: 'VeronicaUquillas 4/19/2011 16:07' prior: 0!
RGObject is the root class of the Ring meta-model.
A ring object can be annotated and knows its environment!

RGObject subclass: #RGDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 1/20/2012 10:05'!
environment
	"If the receiver has not namespace assigned the default is used"
	"self class environment = Smalltalk globals"

	^ self annotationNamed: #environment ifAbsent: [ self class environment ]! !

!RGDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 1/20/2012 10:05'!
environment: namespace

	namespace = self class environment
		ifFalse:[ self annotationNamed: #environment put: namespace ]! !

!RGDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/25/2011 12:40'!
rootEnvironment
	"Retrieves the runtime environment of an RGObject"
	"If the receiver is defined in an RGContainer the runtime environment is the one associated to its container"
	| parent |
	
	parent:= self environment.
	[ parent isRingObject ]
		whileTrue:[ parent:= parent environment ].

	^parent! !

RGDefinition subclass: #RGNamedDefinition
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGNamedDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:08' prior: 0!
RGDefinition is a named ring object.!

RGNamedDefinition subclass: #RGElementDefinition
	instanceVariableNames: 'parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGElementDefinition commentStamp: 'StephaneDucasse 7/26/2011 14:02' prior: 0!
RGElementDefinition is the abstract class for representing elements of a class-alike definition (i.e., methods, variables, comment).

parent holds the RGClassDefinition or RGMetaclassDefinition defining this element.
	
	
Now a RingEntityDefinition offers two APIs: one that is generic and works for all the source code entities and this is the one we just 
presented: parent, parentName and realParent. Having such interface is important to build generic tools that could manipulate 
any entities in a polymorphic way (yes no isKindOf: everywhere).

In addition, a ring method definition offers a specific interface that should only be used when you know that you are solely manipulate
specific entity such as class element: method definition, class comment, and variables. 

Here is the equivalence table

	realParent 				realClass
	parent					ringClass
	parentName			className
	
For example for a methodDefinition we will have the following:

GENERIC API
------------------
* To access the ring class definition name, use parentName
	aRGMethodDefinition parentName
	
Example:
	(Point>>#dist:) asRingDefinition parentName
		->  #Point
		
* If you have a complete model where classes and methods are ring definition, to access the ring class definition , use parent
	aRGMethodDefinition parent
	
Example:
	aRGMethodDefinition(Point>>#dist:) parent
		->  aRGClassDefinition(Point)
		
* If you want to access the smalltalk class that contains the compiledMethod that is represented by a ringMethodDefinition, use realParent
	aRGMethodDefinition realParent
	
Example:
	(Point>>#dist:) asRingDefinition realParent
		->  Point
		


CLASS Element specific API
------------------------------------------
* The message class returns the class of the object :). Yes as you see we could not use class and className because class is already used to refer to the class of the object.

Example:
	(Point>>#dist:) asRingDefinition class
		->  RingMethodDefinition
		
* The message className returns the name of the ring class defining the reingMethodDefinition.

Example:
	(Point>>#dist:) asRingDefinition className
		->  #Point		
		
* If you have a complete model where classes and methods are ring definition, to access the ring class definition , use parent
	aRGMethodDefinition ringClass
	
Example:
	aRGMethodDefinition(Point>>#dist:) ringClass
		->  aRGClassDefinition(Point)
		
		
* If you want to access the smalltalk class that contains the compiledMethod that is represented by a ringMethodDefinition, use realClass
	aRGMethodDefinition realClass
	
Example:
	(Point>>#dist:) asRingDefinition realClass
		->  Point

!

RGElementDefinition subclass: #RGCommentDefinition
	instanceVariableNames: 'content stamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGCommentDefinition commentStamp: 'VeronicaUquillas 5/6/2011 10:54' prior: 0!
RGCommentDefinition is a first-class representation of class's comments!

!RGCommentDefinition class methodsFor: 'instance creation' stamp: 'VeronicaUquillas 9/18/2011 15:41'!
realClass: aClass
	"Creates a ring comment definition from a Smalltalk class"
	^(super realClass: aClass)
		asActive;
		yourself! !

!RGCommentDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/25/2011 22:22'!
<= aRGCommentDefinition
	"Sort comment definition according to: 1) name of the class"
	
	^(self parentName <= aRGCommentDefinition parentName)! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/18/2011 15:40'!
asActive

	"Sets the receiver as active object, which will allow itself to retrieve its data from the class organization"
	self annotationNamed: self class statusKey put: #active.! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/19/2011 16:53'!
asHistorical

	"Sets the receiver as historical object, which will allow itself to retrieve its data using the sourcePointer"
	| realClass |
	self annotationNamed: self class statusKey put: #historical.
	self sourcePointer ifNil:[
		realClass := self realClass.
		realClass notNil ifTrue: [ 
			realClass organization commentRemoteStr
				ifNotNil: [:str | self sourcePointer: str sourcePointer ] ] ]! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/18/2011 14:46'!
asPassive

	"Sets the receiver as passive object, which will allow itself to retrieve its data that was assigned in its creation"
	self annotationNamed: self class statusKey put: #passive! !

!RGCommentDefinition methodsFor: 'converting' stamp: 'VeronicaUquillas 8/26/2011 11:27'!
asStringOrText

	| text |
	self realClass ifNil: [ ^self fullName ].
	text := self fullName asText.
	text addAttribute: TextEmphasis italic.
	^ text! !

!RGCommentDefinition methodsFor: 'stamp values' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
authorAlias 

	^self annotationNamed: self class authorAliasKey
		ifAbsentPut:[ self class parseAuthorAliasFrom: stamp ]! !

!RGCommentDefinition methodsFor: 'stamp values' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
authorAlias: aString 

	self annotationNamed: self class authorAliasKey
			put: aString ! !

!RGCommentDefinition methodsFor: 'to remove as soon as possible' stamp: 'MarianoMartinezPeck 5/7/2012 11:31'!
category
	^self realClass category! !

!RGCommentDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 8/25/2011 14:41'!
commentDataPointers
	"Retrieves the combination key to look for information of the receiver in the source file"
	^'commentStamp:' -> #commentStamp:! !

!RGCommentDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/19/2011 22:03'!
content

	self isActive 
		ifTrue: [ ^ self realClass organization classComment ].
	self isHistorical
		ifTrue: [ ^ self contentAtPointer ifNil:[ self realClass ifNil:[ content ] ifNotNil:[ :rc| rc organization classComment ] ] ].	
	^ content! !

!RGCommentDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/27/2011 16:02'!
content: anObject

	content:= anObject! !

!RGCommentDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 9/18/2011 15:44'!
contentAtPointer
	"A RGCommentDefinition may be created to point the sourceFile in which case it retrieves the class comment"

	^ self sourcePointer notNil
		ifTrue: [ SourceFiles sourceCodeAt: self sourcePointer ]
		ifFalse:[ '' ]! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/19/2011 16:53'!
fromActiveToHistorical
	"If the receiver was generated as an active comment, it can be converted to a historical one by reading the data of the real class (if exists)"
	
	self isActive ifTrue: [
		self asHistorical ]! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/19/2011 16:55'!
fromActiveToPassive
	"If the receiver was generated as an active comment, it can be converted to a passive one by reading the data of the real class organization"
	
	self isActive ifTrue: [
		| realClass |
		realClass := self realClass.
		realClass notNil
			ifTrue: [
				self content: realClass organization classComment.
				self stamp: realClass organization commentStamp ].
		self asPassive ]! !

!RGCommentDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/28/2011 10:13'!
fullName
	"Keeps a unique description for the receiver. As annotation to avoid converting each time is invoked"
	
	^self annotationNamed: self class fullNameKey
		ifAbsentPut:[ (self parentName, ' ', self name) asSymbol ]! !

!RGCommentDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 5/17/2011 13:44'!
hasAuthorAlias

	^self hasStamp and:[ self authorAlias isEmptyOrNil not ]! !

!RGCommentDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/27/2011 10:48'!
hasStamp

	^stamp isEmptyOrNil not

	! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/18/2011 15:41'!
isActive

	"A ring comment isActive when it needs to access the class organization for retrieving its data"
	^(self annotationNamed: self class statusKey) 
		ifNil:[ false ] 
		ifNotNil:[ :status| status == #active ]! !

!RGCommentDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/27/2011 10:41'!
isComment

	^true! !

!RGCommentDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 7/26/2011 14:09'!
isCommentReference

	^ true! !

!RGCommentDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 5/17/2011 09:57'!
isEmptyOrNil

	^content isEmptyOrNil ! !

!RGCommentDefinition methodsFor: 'testing' stamp: 'MarcusDenker 10/6/2012 14:57'!
isFromTrait
	^false! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/18/2011 14:44'!
isHistorical

	"A ring comment can be used to point an old version of the receiver, in this case it will use the sourcePointer to retrieve its information"
	^(self annotationNamed: self class statusKey) 
		ifNil:[ false ] 
		ifNotNil:[ :status| status == #historical ]! !

!RGCommentDefinition methodsFor: 'type of comments' stamp: 'VeronicaUquillas 9/18/2011 14:44'!
isPassive

	"A ring comment isPassive by default.  In this case it will retrieve the data that was assigned in its creation"
	^(self annotationNamed: self class statusKey) 
		ifNil:[ true ] 
		ifNotNil:[ :status| status == #passive ]! !

!RGCommentDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 14:28'!
isSameRevisionAs: aRGCommentDefinition
	"This method look for equality of the properties of the receiver"
	"A comment validates only its contents and not its stamp"
	
	^(super isSameRevisionAs: aRGCommentDefinition)
		and:[ self content = aRGCommentDefinition content ]! !

!RGCommentDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 8/21/2011 18:01'!
isValid
	"for compatibility with method definition"

	^ true! !

!RGCommentDefinition methodsFor: 'to remove as soon as possible' stamp: 'VeronicaUquillas 9/3/2011 20:53'!
methodClass
	self flag: 'if comments are mixed with methods use #realClass instead'.
	^self realClass! !

!RGCommentDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/26/2011 15:25'!
name

	^name ifNil:[ name := #Comment ]! !

!RGCommentDefinition methodsFor: 'printing' stamp: 'VeronicaUquillas 9/1/2011 10:02'!
printOn: aStream 

	self parentName ifNotNil: [
		aStream nextPutAll: self parentName;
				  nextPutAll: ' ' ].
	aStream nextPutAll: self name! !

!RGCommentDefinition methodsFor: 'to remove as soon as possible' stamp: 'VeronicaUquillas 9/3/2011 20:53'!
selector
	self flag: 'if comments are mixed with methods use #name instead'.
	^self name! !

!RGCommentDefinition methodsFor: 'backward compatibility' stamp: 'VeronicaUquillas 8/25/2011 22:37'!
sourceCode

	^ self content   ! !

!RGCommentDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 8/25/2011 14:33'!
sourcePointer
	"Retrieves the sourcePointer for this definition if exists"
	
	^self annotationNamed:  self class sourcePointerKey   ! !

!RGCommentDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 8/25/2011 14:32'!
sourcePointer: aNumber

	self annotationNamed:  self class sourcePointerKey put: aNumber ! !

!RGCommentDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/19/2011 22:04'!
stamp
	"Retrieves the user-alias + timestamp associated to the receiver (if exists)"
	
	self isActive 
		ifTrue: [ ^ self realClass organization commentStamp ].
	self isHistorical
		ifTrue: [ ^ self stampAtPointer ifNil:[ self realClass ifNil:[ stamp ] ifNotNil:[ :rc| rc organization commentStamp ] ] ].	
	^ stamp! !

!RGCommentDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/27/2011 10:42'!
stamp: anObject

	stamp:= anObject! !

!RGCommentDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 9/18/2011 15:44'!
stampAtPointer
	"A RGMethodDefinition may be created to point the sourceFile in which case it retrieves the stamp"

	^ self sourcePointer notNil
		ifTrue: [ SourceFiles timeStampAt: self sourcePointer for: self commentDataPointers ]
		ifFalse:[ nil ]	! !

!RGCommentDefinition methodsFor: 'stamp values' stamp: 'VeronicaUquillas 9/19/2011 11:10'!
timeStamp 

	^ self annotationNamed: self class timeStampKey 
		ifAbsentPut: [ self class 
							parseTimestampFrom: self stamp 
							default: (DateAndTime epoch) ]! !

!RGCommentDefinition methodsFor: 'stamp values' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
timeStamp: aTimestamp 

	self annotationNamed: self class timeStampKey
			put: aTimestamp ! !

!RGElementDefinition class methodsFor: 'parsing stamp' stamp: 'VeronicaUquillas 8/22/2012 15:12'!
basicParseAuthorAliasFrom: aString
	"Parse an alias/name of the author from a string that is extracted from a source file. If there is no alias/name we return emtpy string."
	| tokens dateStartIndex unknown |
	"The following timestamp strings are supported (source: squeak sources archeological survey):
		<authorname><date><time>. 
		<authorname><date>
		<date><time>
		<date><time><authorname>
		<date><authorname>
		<historical>
	All fields can be separated by spaces or line ends but a separator between author alias/name and date can be missing as well"
	"unknown:= 'unknown'."
	unknown:= nil.
	aString isEmptyOrNil ifTrue:[ ^unknown ].
	dateStartIndex := (aString indexOf: $/) - 1. 
	"If there is no / character in the timestamp, no author alias/name exists"
	dateStartIndex = -1 ifTrue:[ ^unknown ].
	 ^[	"Go the start of the date string (there can be 1 or 2 digits and a space separator can be missing at the front!!!!)"
		((dateStartIndex < 2) not and:[(aString at: dateStartIndex - 1) isDigit]) ifTrue:[dateStartIndex := dateStartIndex - 1].
		"Extract only those tokens that do not possible represent date or time - meaning that authorname may be at the end"
		tokens := (aString copyFrom: dateStartIndex to: aString size) substrings reject: [:token | (token occurrencesOf: $/) = 2 or:[(token occurrencesOf: $:) = 1]].
		"only one token should be left if author name/alias exists"
		^tokens isEmpty
			ifTrue:[ "if dateStartIndex is not 1 then the authorname may be at the beginning"
					dateStartIndex > 1
					ifTrue: [ (aString copyFrom: 1 to: dateStartIndex - 1) trimBoth ]
					ifFalse:[ unknown ] ]
			ifFalse:[ tokens first ]
		] 
		on: Exception 
		do:[:e | unknown ]! !

!RGElementDefinition class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 8/16/2011 14:37'!
class: aRGBehaviorDefinition
	"The argument is a ring object and serves as the parent of a method, variable or class comment"

	^self new
		parent: aRGBehaviorDefinition;
		yourself! !

!RGElementDefinition class methodsFor: 'elements-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
classNameKey

	^#className! !

!RGElementDefinition class methodsFor: 'elements-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
isMetaSideKey

	^#isMetaSide! !

!RGElementDefinition class methodsFor: 'parsing stamp' stamp: 'VeronicaUquillas 5/9/2011 14:19'!
parseAuthorAliasFrom: aString
	"Allows other applications  to treat a different empty alias by overriding this method"
	
	^self basicParseAuthorAliasFrom: aString! !

!RGElementDefinition class methodsFor: 'parsing stamp' stamp: 'VeronicaUquillas 5/12/2011 19:58'!
parseTimestampFrom: aString
	
	^self parseTimestampFrom: aString default: nil! !

!RGElementDefinition class methodsFor: 'parsing stamp' stamp: 'VeronicaUquillas 8/22/2012 15:12'!
parseTimestampFrom: aString default: anObject
	"Parse a date-time from a timestamp-string that is extracted from a source file. If there is no timestamp, or we cannot make sense of it, we return the default value."
	| tokens dateStartIndex unknown |
	"The following timestamp strings are supported (source: squeak sources archeological survey):
		<authorname><date><time>. 
		<authorname><date>
		<date><time>
		<date><time><authorname>
		<date><authorname>
		<historical>
	All fields can be separated by spaces or line ends but a separator between authorname and date can be missing as well"
	unknown:= anObject.
	aString isEmptyOrNil ifTrue:[ ^unknown ].
	dateStartIndex := (aString indexOf: $/) - 1. 
	"If there is no / character in the timestamp, we cannot parse a date and return the epoch"
	dateStartIndex = -1 ifTrue:[ ^unknown ].
	^ [	"Go the start of the date string (there can be 1 or 2 digits and a space separator can be missing at the front!!!!)"
		((dateStartIndex < 2) not and:[(aString at: dateStartIndex - 1) isDigit]) ifTrue:[dateStartIndex := dateStartIndex - 1].
		"Extract only those tokens that possibly represent date or time"
		tokens := (aString copyFrom: dateStartIndex to: aString size) substrings select: [:token | (token occurrencesOf: $/) = 2 or:[(token occurrencesOf: $:) = 1]].
		"2 tokens is a datetime"
		tokens size = 2
			ifTrue:[(tokens joinUsing: Character space) asDateAndTime]
			ifFalse:[(tokens first) asDate asDateAndTime]
		] 
		on: Exception 
		do:[:e | unknown ]! !

!RGElementDefinition class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 8/16/2011 14:38'!
realClass: aClass
	"The argument is a Smalltalk class and the parent of a method, variable, class comment.
	However it is not set as parent but only its name and scope (instance/class)"

	^self new
		setParentInfo: aClass;
		yourself! !

!RGElementDefinition class methodsFor: 'elements-annotations' stamp: 'VeronicaUquillas 8/25/2011 22:09'!
sourcePointerKey 

	^#sourcePointer! !

!RGElementDefinition class methodsFor: 'elements-annotations' stamp: 'VeronicaUquillas 9/17/2011 18:04'!
statusKey

	^#statusKey! !

!RGElementDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/31/2011 19:22'!
= aRGElementDefinition
	"This method look for equality of the properties of the receiver"
	"Verifies the class and the parentName of the receiver"

	^self class = aRGElementDefinition class
		and:[ self parentName == aRGElementDefinition parentName 
			and:[ self isMetaSide = aRGElementDefinition isMetaSide ] ]! !

!RGElementDefinition methodsFor: 'backward compatibility' stamp: 'VeronicaUquillas 9/1/2011 15:07'!
actualClass
	"returns the Smalltalk class of the receiver"
	
	^ self realClass! !

!RGElementDefinition methodsFor: 'tools' stamp: 'BenjaminVanRyseghem 2/8/2012 17:10'!
browse

	^ self systemNavigation browseClass: self realClass! !

!RGElementDefinition methodsFor: 'class element specific api' stamp: 'StephaneDucasse 7/26/2011 14:02'!
className
	
	^ self parentName! !

!RGElementDefinition methodsFor: 'class element specific api' stamp: 'StephaneDucasse 7/26/2011 14:21'!
className: aName
	
	^ self parentName: aName! !

!RGElementDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/1/2011 15:50'!
fullName: aString

	^ self annotationNamed: self class fullNameKey put: aString asSymbol! !

!RGElementDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/31/2011 19:34'!
hash
	"Hash is re-implemented because #= is re-implemented"
	
	^self class hash bitXor: (self parentName hash bitXor: self isMetaSide hash)! !

!RGElementDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/19/2011 14:21'!
isDefined
	"isDefined when the receiver has its realClass defined in the system"

	^self realClass notNil! !

!RGElementDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 7/26/2011 14:01'!
isMetaSide
	"Even thought several class elements do not define this property (ie. class variables, pool variables) they understand it"
	"This is a derived property from the class definining the receiver and thus its value is kept as an annotation"
	"Default value is false"

	^self annotationNamed: self class isMetaSideKey ifAbsentPut: [ false ]! !

!RGElementDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 7/26/2011 14:01'!
isMetaSide: aBoolean

	self annotationNamed: self class isMetaSideKey put: aBoolean! !

!RGElementDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 14:28'!
isSameRevisionAs: aRGElementDefinition
	"This method look for equality of the properties of the receiver"
	"Verifies the class and the parentName of the receiver"

	^self class = aRGElementDefinition class
		and:[ self parentName == aRGElementDefinition parentName ]! !

!RGElementDefinition methodsFor: 'generic parent api' stamp: 'StephaneDucasse 7/26/2011 14:00'!
parent
	"The parent of a class definition element: method, comment and variable is the class definition. This method retrieves the class that defines such element"
	
	^ parent! !

!RGElementDefinition methodsFor: 'generic parent api' stamp: 'StephaneDucasse 7/26/2011 14:00'!
parent: aRGBehaviorDefinition
	"Set the class associated to the receiver"

	parent := aRGBehaviorDefinition.
	self setParentInfo: aRGBehaviorDefinition.! !

!RGElementDefinition methodsFor: 'generic parent api' stamp: 'StephaneDucasse 7/26/2011 14:01'!
parentName
	"Retrieves the name of the class defining the receiver. Its value is kept as an annotation"

	^ self annotationNamed: self class classNameKey! !

!RGElementDefinition methodsFor: 'generic parent api' stamp: 'VeronicaUquillas 8/31/2011 17:31'!
parentName: aString

	self annotationNamed: self class classNameKey put: aString asSymbol! !

!RGElementDefinition methodsFor: 'class element specific api' stamp: 'StephaneDucasse 7/26/2011 14:02'!
realClass
	"Retrieves the Class/Trait/.. object in the System corresponding to the class of the this element."
	
	^ self realParent! !

!RGElementDefinition methodsFor: 'generic parent api' stamp: 'StephaneDucasse 7/26/2011 13:58'!
realParent
	"Retrieves the Class/Trait/.. object in the System corresponding to the class of the this element."
	
	^self parent notNil
		ifTrue: [ self parent realClass ]
		ifFalse: [ self rootEnvironment classNamed: self parentName ]! !

!RGElementDefinition methodsFor: 'class element specific api' stamp: 'StephaneDucasse 7/26/2011 13:59'!
ringClass
	"Return the ring definition of the class containing the receiver."
	
	^ self parent! !

!RGElementDefinition methodsFor: 'private' stamp: 'VeronicaUquillas 8/16/2011 14:33'!
setParentInfo: anObject
	"anObject is aRGBehaviorDefinition or aClass/aTrait"

	self parentName: anObject name. 
	self isMetaSide: anObject isMeta! !

!RGElementDefinition methodsFor: 'class element specific api' stamp: 'VeronicaUquillas 8/25/2011 22:47'!
theNonMetaClassName

	^self theNonMetaParentName ! !

!RGElementDefinition methodsFor: 'generic parent api' stamp: 'VeronicaUquillas 5/4/2011 13:19'!
theNonMetaParentName
	"Rejects the prefix ' class' or ' classTrait' of the parentName"
	| index |
	
	index := self parentName
				indexOfSubCollection: ' class'
				startingAt: 1
				ifAbsent: [ ^self parentName ].

	^(self parentName 
		copyFrom: 1
		to: index - 1) asSymbol! !

RGElementDefinition subclass: #RGMethodDefinition
	instanceVariableNames: 'protocol sourceCode stamp package origin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGMethodDefinition commentStamp: 'VeronicaUquillas 9/18/2011 16:00' prior: 0!
RGMethodDefinition is a concrete representation of methods. It can be used to build browser for methods that are not in the image. It is polymorphic with CompiledMethod. 

* We can ask a RGMethodDefinition for its selector using the selector message.
Example:
	(Point>>#dist:) asRingDefinition selector
		-> #dist

We can also ask the ring object representation of its class or the Smalltalk class actually implementing the corresponding compiledMethod. 

* To access the ring class definition name, use parentName
	aRGMethodDefinition parentName
	
Example:
	(Point>>#dist:) asRingDefinition parentName
		->  #Point
		
* If you have a complete model where classes and methods are ring definition, to access the ring class definition , use parent
	aRGMethodDefinition parent
	
Example:
	aRGMethodDefinition(Point>>#dist:) parent
		->  aRGClassDefinition(Point)
		
* If you want to access the smalltalk class that contains the compiledMethod that is represented by a ringMethodDefinition, use realParent
	aRGMethodDefinition realParent
	
Example:
	(Point>>#dist:) asRingDefinition realParent
		->  Point
		

Now a RingEntityDefinition offers two APIs: one that is generic and works for all the source code entities and this is the one we just 
presented: parent, parentName and realParent. Having such interface is important to build generic tools that could manipulate 
any entities in a polymorphic way (yes no isKindOf: everywhere).

In addition, a ring method definition offers a specific interface that should only be used when you know that you are solely manipulate
specific entity such as class element: method definition, class comment, and variables. 

Here is the equivalence table

	realParent 				realClass
	parent					ringClass
	parentName			className


* The message class returns the class of the object :).

Example:
	(Point>>#dist:) asRingDefinition class
		->  RingMethodDefinition
		
* The message className returns the name of the ring class defining the reingMethodDefinition.

Example:
	(Point>>#dist:) asRingDefinition className
		->  #Point		
		
* If you have a complete model where classes and methods are ring definition, to access the ring class definition , use parent
	aRGMethodDefinition ringClass
	
Example:
	aRGMethodDefinition(Point>>#dist:) ringClass
		->  aRGClassDefinition(Point)
		
		
* If you want to access the smalltalk class that contains the compiledMethod that is represented by a ringMethodDefinition, use realClass
	aRGMethodDefinition realClass
	
Example:
	(Point>>#dist:) asRingDefinition realClass
		->  Point

!

!RGMethodDefinition class methodsFor: 'instance creation' stamp: 'VeronicaUquillas 8/16/2011 14:18'!
class: aRGBehaviorDefinition selector: aString
	"Create a ring method definition from a ring class and a selector"

	^(self class: aRGBehaviorDefinition)
		selector: aString asSymbol;
		yourself! !

!RGMethodDefinition class methodsFor: 'elements-annotations' stamp: 'VeronicaUquillas 7/22/2011 11:23'!
isExtensionKey 

	^#isExtension! !

!RGMethodDefinition class methodsFor: 'instance creation' stamp: 'VeronicaUquillas 9/18/2011 15:08'!
realClass: aClass selector: aString
	"Creates a ring method definition from a Smalltalk class and a selector <compiledMethod>"

	^(aClass>>aString asSymbol) asActiveRingDefinition! !

!RGMethodDefinition class methodsFor: 'categories' stamp: 'VeronicaUquillas 4/20/2011 17:25'!
unclassifiedProtocolName
	"Return the string labeling the unclassified protocol."

	^ 'as yet unclassified'! !

!RGMethodDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/25/2011 22:26'!
<= aRGMethodDefinition
	"Sort method definitions according to: 1. class name - 2. class comment - 3. method selector - 4. timestamp"

	self parentName < aRGMethodDefinition parentName ifTrue: [^true].
	self parentName > aRGMethodDefinition parentName ifTrue: [^false].

	self flag: 'This ugly test is needed right now because the old system is used to represent 
	class comment as method with Comment selector. And to mix comment and methods'.
	aRGMethodDefinition isComment ifTrue: [^false].
	
	^(self selector < aRGMethodDefinition selector) 
 		  or: [ (self selector == aRGMethodDefinition selector) and: [
    			  self timeStamp <= aRGMethodDefinition timeStamp ]]! !

!RGMethodDefinition methodsFor: 'comparing' stamp: 'StephaneDucasse 8/12/2011 11:52'!
= aRGMethodDefinition
	"This method look for equality of the key properties of the receiver"

	^(super = aRGMethodDefinition)
		and: [ self selector == aRGMethodDefinition selector ]! !

!RGMethodDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 4/20/2011 17:35'!
addInContainer: aRGContainer

	aRGContainer addMethod: self
	! !

!RGMethodDefinition methodsFor: 'traits' stamp: 'mada 5/5/2012 13:07'!
argumentNames
	"Return an array with the argument names of the method's selector"

	^ self compiledMethod argumentNames! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'VeronicaUquillas 9/18/2011 15:30'!
asActive

	"Sets the receiver as active object, which will allow itself to retrieve its data from the compiled method"
	self annotationNamed: self class statusKey put: #active! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'VeronicaUquillas 9/19/2011 15:42'!
asHistorical

	"Sets the receiver as historical object, which will allow itself to retrieve its data using the sourcePointer"
	self annotationNamed: self class statusKey put: #historical.
	self sourcePointer ifNil:[ | pointer compiledMethod |
		pointer := 0.
		compiledMethod := self compiledMethod.
		compiledMethod notNil ifTrue: [
			pointer := compiledMethod sourcePointer ].
		pointer isZero
			ifFalse:[ self sourcePointer: pointer ] ]! !

!RGMethodDefinition methodsFor: 'conversion' stamp: 'CamilloBruni 7/4/2012 01:08'!
asMCMethodDefinition
   "Creates a MCMethodDefinition from the receiver"
	^ MCMethodDefinition cachedDefinitions
		at: self compiledMethod
		ifAbsentPut: [ self basicAsMCMethodDefinition ].! !

!RGMethodDefinition methodsFor: 'conversion' stamp: 'CamilloBruni 2/28/2012 14:28'!
asMCMethodDefinitionFromFile: file
	"Creates a MCMethodDefinition from a source file assuming the current metod isActive.
	This is significantly faster than the asMCMethodDefinition as it does not open the source file pointer twice; for the sourceCode and the timeStamp"
	| compiledMethod methodClass|
	compiledMethod := self compiledMethod.
	methodClass := compiledMethod methodClass.
	
   ^ MCMethodDefinition
		className: methodClass theNonMetaClass name
 	   	classIsMeta: methodClass isMeta
		selector: compiledMethod selector
		category: compiledMethod category
		timeStamp:( compiledMethod timeStampFromFile: file)
		source: (compiledMethod getSourceFromFile: file)! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'abc 9/18/2011 16:24'!
asPassive

	"Sets the receiver as passive object, which will allow itself to retrieve its data that was assigned in its creation"
	self annotationNamed: self class statusKey put: #passive! !

!RGMethodDefinition methodsFor: 'conversion' stamp: 'StephaneDucasse 9/17/2011 12:00'!
asStringOrText

	| text |
	self realClass ifNil: [ ^self fullName ].
	text := (self fullName,' {', (self protocol ifNil: ''), '} ') asText.
	self isLocalSelector
		ifFalse: [ text addAttribute: TextEmphasis italic ].
	^ text! !

!RGMethodDefinition methodsFor: 'stamp values' stamp: 'StephaneDucasse 8/12/2011 11:57'!
authorAlias 

	^self 
		annotationNamed: self class authorAliasKey
		ifAbsentPut: [ self class parseAuthorAliasFrom: self stamp ]! !

!RGMethodDefinition methodsFor: 'stamp values' stamp: 'StephaneDucasse 8/12/2011 11:57'!
authorAlias: aString 

	self annotationNamed: self class authorAliasKey put: aString ! !

!RGMethodDefinition methodsFor: 'conversion' stamp: 'CamilloBruni 2/28/2012 20:15'!
basicAsMCMethodDefinition
   "Creates a MCMethodDefinition from the receiver"
   self isActive ifTrue: [ 
		self compiledMethod sourceFileStream ifNotNilDo: [ :file|
			^ self asMCMethodDefinitionFromFile: file]].
	
   ^ MCMethodDefinition
		className: self theNonMetaParentName
 	   	classIsMeta: self isMetaSide
		selector: self selector
		category: self protocol
		timeStamp: self stamp
		source: self sourceCode! !

!RGMethodDefinition methodsFor: 'tools' stamp: 'BenjaminVanRyseghem 2/8/2012 17:10'!
browse

	^ Smalltalk tools browser
		fullOnClass: self realClass 
		selector: (self isDefined ifTrue: [ self selector ] ifFalse: [ nil ])! !

!RGMethodDefinition methodsFor: 'backward compatibility' stamp: 'StephaneDucasse 8/21/2011 19:10'!
category

	^ self protocol! !

!RGMethodDefinition methodsFor: 'to remove as soon as possible' stamp: 'VeronicaUquillas 8/31/2011 13:57'!
classIsMeta

	^self isMetaSide! !

!RGMethodDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 7/26/2011 14:06'!
classSymbol
	self flag: #DONOTUSEITPLEASE.
	"we will deprecate it" 
	^ self className! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:52'!
compiledMethod
	"Retrieves the compiled method of this definition if exists"
	
	| rClass |
	(rClass := self realClass) notNil
		ifTrue: [ (rClass includesSelector: self selector)
			ifTrue: [ ^rClass >> self selector ] ].
			
	^nil! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'StephaneDucasse 5/15/2012 13:12'!
fileIndex
	^SourceFiles fileIndexFromSourcePointer: self sourcePointer! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'StephaneDucasse 5/13/2012 21:37'!
filePosition
	
	^SourceFiles filePositionFromSourcePointer: self sourcePointer! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'VeronicaUquillas 9/19/2011 15:43'!
fromActiveToHistorical
	"If the receiver was generated as an active method, it can be converted to a historical one by reading the data of the compiled method (if exists)"
	
	self isActive ifTrue: [
		self asHistorical ]! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'MarcusDenker 4/29/2012 10:34'!
fromActiveToPassive
	"If the receiver was generated as an active method, it can be converted to a passive one by reading the data of the compiled method (if exists)"
	
	self isActive ifTrue: [
		| compiledMethod |
		compiledMethod := self compiledMethod.
		compiledMethod notNil
			ifTrue: [
				self protocol: compiledMethod category.
				self sourceCode: compiledMethod sourceCode.
				self stamp: compiledMethod timeStamp ].
		self asPassive ]! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:52'!
fullName
	"Keeps a unique description for the receiver. As annotation to avoid converting each time is invoked"
	
	^self annotationNamed: self class fullNameKey
		ifAbsentPut: [ (self parentName, '>>', self selector) asSymbol ]! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'StephaneDucasse 5/15/2012 13:17'!
getPreambleFrom: aFileStream at: position
	|  writeStream |
	writeStream := String new writeStream.
	position to: 0 by: -1 do: [ :p | 
			| c | 
			aFileStream position: p.
			c := aFileStream basicNext.
			c == $!!
				ifTrue: [^ writeStream contents reversed]
				ifFalse: [writeStream nextPut: c]]! !

!RGMethodDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 3/19/2011 17:10'!
hasStamp

	^stamp isEmptyOrNil not! !

!RGMethodDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/2/2011 11:28'!
hash

	^super hash bitXor: self selector hash! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'VeronicaUquillas 9/18/2011 15:39'!
isActive

	"A ring method isActive when it needs to access the compiledMethod for retrieving its data"
	^(self annotationNamed: self class statusKey) 
		ifNil:[ false ] 
		ifNotNil:[ :status| status == #active ]! !

!RGMethodDefinition methodsFor: 'traits' stamp: 'MarcusDenker 3/24/2011 14:51'!
isBinarySelector
	^self selector
		allSatisfy: [:each | each isSpecial]! !

!RGMethodDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 7/26/2011 14:13'!
isClassMethodReference

	^ self isMetaSide ! !

!RGMethodDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 7/26/2011 14:07'!
isCommentReference

	^ false! !

!RGMethodDefinition methodsFor: 'testing' stamp: 'StephaneDucasse 8/12/2011 11:57'!
isDefined
	"Answer whether the receiver exists in the environment"
	
	| rClass |
	self selector isDoIt ifTrue:[ ^false ].
	rClass := self realClass.
	^rClass notNil and: [ rClass includesSelector: self selector ]! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/22/2011 15:53'!
isExtension
	"The receiver is an extension when is defined in a different package to the one of its parent.
	Ring allows to set this property. If not assigned tries to find its value "
	
	^self 
		annotationNamed: self class isExtensionKey
		ifAbsent: [  
			(self parent notNil and:[ self package notNil ])
				ifTrue: [ | value |
					       value :=  self parent package ~= self package.
						 self annotationNamed: self class isExtensionKey put: value.
						 value ]
				ifFalse: [ self protocol ifNil:[ false ] ifNotNil:[ :prot|  prot beginsWith: '*' ] ] ]! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:53'!
isExtension: aBoolean
	"Explicitily set that the receiver is an extension"
	
	self annotationNamed: self class isExtensionKey put: aBoolean ! !

!RGMethodDefinition methodsFor: 'testing' stamp: 'CamilloBruni 9/22/2012 21:16'!
isFromTrait

	"Return true for methods that have been included from Traits"
	^ self origin isTrait and: [ self origin ~= self methodClass ]! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'VeronicaUquillas 9/18/2011 14:44'!
isHistorical

	"A ring method can be used to point an old version of the receiver, in this case it will use the sourcePointer to retrieve its information"
	^(self annotationNamed: self class statusKey) 
		ifNil:[ false ] 
		ifNotNil:[ :status| status == #historical ]! !

!RGMethodDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 4/20/2011 17:35'!
isIncludedInContainer: aRGContainer

	^aRGContainer includesMethod: self! !

!RGMethodDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 8/21/2011 19:09'!
isLocalSelector
	"Answer whether the receiver exists in the environment as a local implementation"
	
	| rClass |
	rClass:= self realClass.
	^rClass notNil and: [ rClass includesLocalSelector: self selector ]! !

!RGMethodDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 3/21/2011 17:51'!
isMethod

	^true! !

!RGMethodDefinition methodsFor: 'type of methods' stamp: 'VeronicaUquillas 9/18/2011 15:38'!
isPassive

	"A ring method isPassive when it retrieves the data that was assigned in its creation.
	By default is passive"
	^(self annotationNamed: self class statusKey) 
		ifNil:[ true ] 
		ifNotNil:[ :status| status == #passive ]! !

!RGMethodDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 8/25/2011 22:30'!
isSameRevisionAs: aRGMethodDefinition
	"This method look for equality of the properties of the receiver"
	"Stamp is ignored as in many cases a method is saved without containing any change. However it appears as changed due to a different stamp"
	self flag: 'needs to be evaluated'.

	^(super isSameRevisionAs: aRGMethodDefinition)
		and: [ self selector = aRGMethodDefinition selector 
			and: [ self protocol = aRGMethodDefinition protocol
				and: [ self sourceCode = aRGMethodDefinition sourceCode ] ] ]! !

!RGMethodDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 8/21/2011 18:00'!
isValid
	"verifies that the receiver is locally defined in the class and that is not a DoIt"

	^self isDefined! !

!RGMethodDefinition methodsFor: 'backward compatibility' stamp: 'VeronicaUquillas 8/31/2011 15:32'!
method

	^self compiledMethod! !

!RGMethodDefinition methodsFor: 'backward compatibility' stamp: 'StephaneDucasse 8/9/2011 17:42'!
methodClass
	"Return the class to which the receiver belongs to."
	
	^ self realClass! !

!RGMethodDefinition methodsFor: 'to remove as soon as possible' stamp: 'StephaneDucasse 7/26/2011 14:14'!
methodSymbol
	
	^ self selector! !

!RGMethodDefinition methodsFor: 'metrics' stamp: 'VeronicaUquillas 12/17/2011 08:55'!
numberOfLinesOfCode

	^ self annotationNamed: #numberOfLinesOfCode ifAbsentPut: [ self sourceCode lineCount ]! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'CamilloBruni 9/22/2012 21:19'!
origin
	"Return the real oring of this method."
	^ self compiledMethod 
		ifNil: [ self methodClass ]
		ifNotNil: [ :compileMethod| compileMethod origin ]! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 12:40'!
package
	"Retrieves the package in which this class is contained, if exists"
	
	^package! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/20/2011 17:35'!
package: aRGPackage

	package:= aRGPackage! !

!RGMethodDefinition methodsFor: 'printing' stamp: 'StephaneDucasse 8/12/2011 11:56'!
printOn: aStream 

	self parentName ifNotNil: [
		aStream nextPutAll: self parentName;
				  nextPutAll: '>>' ].
	aStream nextPutAll: self selector! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/19/2011 22:03'!
protocol

	self isActive 
		ifTrue: [ ^ self compiledMethod category ].
	self isHistorical
		ifTrue: [ ^ self protocolAtPointer ifNil:[ self compiledMethod ifNil:[ protocol ] ifNotNil:[ :cm| cm category ] ] ].
	^ protocol			! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:53'!
protocol: anObject

	protocol := anObject! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 9/18/2011 15:44'!
protocolAtPointer

	"A RGMethodDefinition that was set as historical will retrieve the protocol using the sourcePointer"

	^ self sourcePointer notNil
		ifTrue: [ SourceFiles protocolAt: self sourcePointer ]
		ifFalse:[ nil ]! !

!RGMethodDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 4/20/2011 17:35'!
removeFromContainer: aRGContainer

	aRGContainer removeMethod: self! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/28/2011 15:54'!
selector
	"Retrieves the name of the method"

	^name! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:53'!
selector: aSymbol
	"The name of a method is known as #selector"

	name := aSymbol! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'MarcusDenker 4/29/2012 10:35'!
sourceCode

	self isActive 
		ifTrue: [ ^ self compiledMethod sourceCode ].
	self isHistorical
		ifTrue: [ ^ self sourceCodeAtPointer ifNil:[ self compiledMethod ifNil:[ sourceCode ] ifNotNil:[ :cm| cm sourceCode ] ] ].	
	^ sourceCode! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:53'!
sourceCode: anObject

	sourceCode := anObject! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 9/18/2011 15:45'!
sourceCodeAtPointer

	"A RGMethodDefinition that was set as historical will retrieve the sourceCode using the sourcePointer"

	^ self sourcePointer notNil
		ifTrue: [ SourceFiles sourceCodeAt: self sourcePointer ]
		ifFalse:[ nil ]! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
sourcePointer
	"Retrieves the sourcePointer for this definition if exists"
	
	^self annotationNamed:  self class sourcePointerKey   ! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'StephaneDucasse 8/1/2011 19:35'!
sourcePointer: aNumber

	self annotationNamed:  self class sourcePointerKey put: aNumber ! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/19/2011 22:02'!
stamp

	self isActive 
		ifTrue: [ ^ self compiledMethod timeStamp ].
	self isHistorical
		ifTrue: [ ^ self stampAtPointer ifNil:[ self compiledMethod ifNil:[ stamp ] ifNotNil:[ :cm| cm timeStamp ] ] ].
	^ stamp! !

!RGMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 8/12/2011 11:53'!
stamp: anObject
	"stores an author alias and a timestamp"
	
	stamp := anObject! !

!RGMethodDefinition methodsFor: 'source pointers' stamp: 'VeronicaUquillas 9/18/2011 15:45'!
stampAtPointer	

	"A RGMethodDefinition that was set as historical will retrieve the stamp using the sourcePointer"

	^ self sourcePointer notNil
		ifTrue: [ SourceFiles timeStampAt: self sourcePointer ]
		ifFalse:[ nil ]! !

!RGMethodDefinition methodsFor: 'stamp values' stamp: 'VeronicaUquillas 9/19/2011 11:06'!
timeStamp 

	^self annotationNamed: self class timeStampKey 
		ifAbsentPut: [ self class 
							parseTimestampFrom: self stamp 
							default: (DateAndTime epoch) ]! !

!RGMethodDefinition methodsFor: 'stamp values' stamp: 'StephaneDucasse 8/12/2011 11:57'!
timeStamp: aTimestamp 

	self annotationNamed: self class timeStampKey put: aTimestamp ! !

RGElementDefinition subclass: #RGVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

RGVariableDefinition subclass: #RGClassInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGClassInstanceVariableDefinition methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 2/15/2011 15:47'!
initialize 

	super initialize.
	self isMetaSide: true.! !

!RGClassInstanceVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/16/2010 14:46'!
isClassInstanceVariable

	^true! !

RGVariableDefinition subclass: #RGClassVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGClassVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/16/2010 14:46'!
isClassVariable

	^true! !

RGVariableDefinition subclass: #RGInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGInstanceVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/16/2010 14:46'!
isInstanceVariable

	^true! !

RGVariableDefinition subclass: #RGPoolVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGPoolVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 2/22/2011 17:49'!
isPoolVariable

	^true! !

!RGVariableDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/2/2011 13:03'!
= aRGVariableDefinition
	"This variable look for equality of the key properties of the receiver"

	^(super = aRGVariableDefinition)
		and:[ self name == aRGVariableDefinition name ]! !

!RGVariableDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/16/2011 16:26'!
environment

	^ self parent environment ! !

!RGVariableDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/28/2011 10:14'!
fullName
	"Keeps a unique description for the receiver. As annotation to avoid converting each time is invoked"
	
	^self annotationNamed: self class fullNameKey
		ifAbsentPut:[ (self parentName, '->', self name) asSymbol ]! !

!RGVariableDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/2/2011 13:03'!
hash

	^super hash bitXor: self name hash! !

!RGVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/16/2010 14:47'!
isClassInstanceVariable

	^false! !

!RGVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/16/2010 14:47'!
isClassVariable

	^false! !

!RGVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 2/15/2011 13:57'!
isGlobalVariable

	^false! !

!RGVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/16/2010 14:47'!
isInstanceVariable

	^false! !

!RGVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 2/22/2011 17:52'!
isPoolVariable

	^false! !

!RGVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/19/2010 12:00'!
isVariable

	^true! !

RGNamedDefinition subclass: #RGGlobalDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGGlobalDefinition commentStamp: 'VeronicaUquillas 4/19/2011 15:52' prior: 0!
An RGGlobalDefinition is an abstract superclass for representing classes, global variables and shared pools
!

RGGlobalDefinition subclass: #RGBehaviorDefinition
	instanceVariableNames: 'superclass methods protocols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGBehaviorDefinition commentStamp: 'VeronicaUquillas 5/9/2011 14:30' prior: 0!
An RGBehaviorDefinition is an abstract definition for class-alike entities (e.g. classes, traits)

Instance Variables
	methods:		<Collection>
	protocols:		<Collection>
	superclass:		<Object>!

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
allSubclassesKey

	^#allSubclasses! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
allSuperclassesKey

	^#allSuperclasses! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
definitionSourceKey

	^#definitionSource! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 8/2/2011 13:35'!
isPoolKey

	^#isPool! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
subclassesKey

	^#subclasses! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
superclassNameKey
	
	^#superclassName! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
superclassesKey

	^#superclasses! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 6/28/2011 15:17'!
traitCompositionSourceKey

	^#traitCompositionSource! !

!RGBehaviorDefinition class methodsFor: 'class-annotations' stamp: 'VeronicaUquillas 8/2/2011 13:43'!
usersKey

	^#users! !

!RGBehaviorDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 5/9/2011 14:34'!
addInContainer: aRGContainer

	aRGContainer addClass: self! !

!RGBehaviorDefinition methodsFor: 'adding/removing methods' stamp: 'VeronicaUquillas 4/29/2011 09:21'!
addMethod: aRGMethodDefinition
	"Adds aRGMethodDefinition in the methods dictionary.
	 Adds the protocol of such method too"

	self addMethod: aRGMethodDefinition in: methods.
	self addProtocol: aRGMethodDefinition protocol! !

!RGBehaviorDefinition methodsFor: 'private' stamp: 'VeronicaUquillas 6/6/2011 10:47'!
addMethod: aRGMethodDefinition in: aCollection
	"Adds aRGMethodDefinition in the collection received"

	aRGMethodDefinition parent ifNil:[ aRGMethodDefinition parent: self ].
	aCollection at: aRGMethodDefinition selector
		put: aRGMethodDefinition! !

!RGBehaviorDefinition methodsFor: 'adding/removing protocols' stamp: 'VeronicaUquillas 5/13/2011 23:42'!
addProtocol: anObject
	"Adds a protocol named anObject.
	Protocols are not repeated"
	
	anObject ifNil:[ ^self ].
	protocols add: anObject! !

!RGBehaviorDefinition methodsFor: 'adding/removing methods' stamp: 'VeronicaUquillas 4/28/2011 15:58'!
addSelector: selectorName classified: protocolName sourced: source

	self addMethod: ((self factory createMethodNamed: selectorName parent: self)
							protocol: protocolName;
							sourceCode: source;
							yourself)! !

!RGBehaviorDefinition methodsFor: 'subclassing' stamp: 'VeronicaUquillas 9/16/2011 16:49'!
addSubclass: aRGBehaviorDefinition
	"Adds a direct subclass of the receiver"

	(self subclasses includes: aRGBehaviorDefinition)
		ifFalse:[ self subclasses add: aRGBehaviorDefinition ]! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 3/21/2011 17:14'!
allSelectors
	"Retrieves all the selectos of the receiver in the chain hierarchy"
	| class selectors |
	
	class:= self.
	selectors := Set new.
	[class notNil] whileTrue: 
			[selectors addAll: class selectors.
			class := class superclass ].
	^selectors! !

!RGBehaviorDefinition methodsFor: 'accessing class hierarchy' stamp: 'VeronicaUquillas 7/28/2011 15:59'!
allSubclasses
	"Retrieves all the subclasses of the receiver in the chan hiearchy - value is kept as an annotation"
	"is a good idea storing this?"
	
	^self annotationNamed: self class allSubclassesKey
		ifAbsentPut:[ | collection index |
			index := 1.
			collection := OrderedCollection withAll: self subclasses.
			[index <= collection size] whileTrue: 
				[collection addAll: (collection at: index) subclasses.
				index := index + 1].
			collection ]! !

!RGBehaviorDefinition methodsFor: 'accessing class hierarchy' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
allSubclasses: aCollection
	"Stores all the subclasses (direct and indirect) as an annotation"
	
	self annotationNamed: self class allSubclassesKey
		ifAbsentPut:[ aCollection ]! !

!RGBehaviorDefinition methodsFor: 'accessing class hierarchy' stamp: 'VeronicaUquillas 6/28/2011 15:23'!
allSuperclasses
	"Answer an OrderedCollection of the receiver's and the receiver's  
	ancestor's superclasses"
	"Is implementation of Behavior more efficient?"
	
	^self annotationNamed: self class allSuperclassesKey
		ifAbsentPut:[ | supers sprClass |
			supers := OrderedCollection new.
			sprClass := self superclass.
			[sprClass notNil] whileTrue: 
				[supers add: sprClass.
				sprClass := sprClass superclass].
			supers ]! !

!RGBehaviorDefinition methodsFor: 'accessing class hierarchy' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
allSuperclasses: aCollection
	"Stores all the superclasses (direct and indirect) as an annotation"
	
	self annotationNamed: self class allSuperclassesKey
		ifAbsentPut:[ aCollection ]! !

!RGBehaviorDefinition methodsFor: 'tools' stamp: 'BenjaminVanRyseghem 2/8/2012 17:10'!
browse

	^ Smalltalk tools browser
		fullOnClass: self realClass selector: nil! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 4/20/2011 17:31'!
compiledMethodNamed: selector
	"Retrieves the compiled method from aRGMethodDefinition"
	| method |

	^(method:= self methodNamed: selector) notNil
	ifTrue: [ method compiledMethod ]
	ifFalse:[ nil ]! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:21'!
definitionSource
	"Retrieves the definition template of the receiver -> aString.
	This value is kept as an annotation"
	
	^self annotationNamed: self class definitionSourceKey! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:23'!
definitionSource: aString
	"Sets the definition template of the receiver -> aString.
	It's stored as an annotation"
	
	self annotationNamed: self class definitionSourceKey
		put: aString! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 8/8/2011 12:47'!
extensionMethods

	^self methods select:[ :each | each isExtension ]! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 9/19/2011 16:51'!
hasMetaclass

	^ false! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/13/2011 16:40'!
hasMethods
	"validates the existance of methods"

	^methods notEmpty! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 3/21/2011 17:03'!
hasProtocols
	"Validates the existance of protocols"

	^protocols notEmpty! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/24/2010 16:40'!
hasSuperclass

	^superclass notNil! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 3/22/2011 16:38'!
hasTraitComposition

	^self traitCompositionSource ~= '{}'! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/13/2011 17:15'!
includesProtocol: aString
	"Looks for a protocols named = aString"
	
	^(protocols detect:[ :each| each = aString ] ifNone:[ nil ]) notNil
! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 4/13/2011 23:54'!
includesSelector: selector
	"Looks if selector is a key in the methods dictionary"
	
	^methods includesKey: selector asSymbol! !

!RGBehaviorDefinition methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 5/12/2011 10:33'!
initialize

	super initialize.
	methods:= IdentityDictionary new.
	protocols:= Set new.! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/13/2011 16:41'!
isDefined
	"If the class exists in the environment"
	
	^self realClass notNil
	! !

!RGBehaviorDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 5/9/2011 14:34'!
isIncludedInContainer: aRGContainer

	^aRGContainer includesClass: self! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: ''!
isMeta
	"By default is considered a non-meta class"

	^false! !

!RGBehaviorDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 14:30'!
isSameRevisionAs: aRGBehaviorDefinition
	"This method look for equality of the properties of the receiver"

	^self class = aRGBehaviorDefinition class
		and:[ self name == aRGBehaviorDefinition name ]! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 4/20/2011 17:31'!
methodNamed: selector
	"Retrieves aRGMethodDefinition that matches the selector given as argument"
	"RB defines methodFor:"

	^methods at: selector asSymbol ifAbsent:[ nil ]! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/17/2010 11:01'!
methods

	^methods! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/2/2011 16:24'!
methods: aDictionary

	methods:= aDictionary! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 4/13/2011 23:51'!
methodsInProtocol: aString
	"Retrieves the methods classified in protocol named aString"

	^methods select:[ :each| each protocol = aString ]! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: ''!
package
	
	self subclassResponsibility! !

!RGBehaviorDefinition methodsFor: 'printing' stamp: 'VeronicaUquillas 11/17/2010 14:31'!
printOn: aStream 
	aStream nextPutAll: self name! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/13/2011 16:43'!
protocols
	"retrieves the protocols of the class"

	^protocols! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 3/21/2011 16:58'!
protocols: aCollection
	"set the protocols of the class"

	protocols:= aCollection! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: ''!
realClass
	"Retrieves the current class existing in the runtime environment"

	^self rootEnvironment classNamed: self name ! !

!RGBehaviorDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 5/9/2011 14:34'!
removeFromContainer: aRGContainer

	aRGContainer removeClass: self! !

!RGBehaviorDefinition methodsFor: 'adding/removing methods' stamp: 'VeronicaUquillas 4/29/2011 13:58'!
removeMethod: aRGMethodDefinition
	"Removes aRGMethodDefinition from the methods dictionary"

	self removeMethod: aRGMethodDefinition from: methods! !

!RGBehaviorDefinition methodsFor: 'private' stamp: 'VeronicaUquillas 4/29/2011 13:58'!
removeMethod: aRGMethodDefinition from: aCollection
	"Removes aRGMethodDefinition from the collection received"

	aCollection removeKey: aRGMethodDefinition selector ifAbsent:[]! !

!RGBehaviorDefinition methodsFor: 'adding/removing protocols' stamp: 'VeronicaUquillas 4/29/2011 09:29'!
removeProtocol: aString
	"Removes a protocol named aString (if exists)"

	protocols remove: aString ifAbsent:[]! !

!RGBehaviorDefinition methodsFor: 'adding/removing methods' stamp: 'VeronicaUquillas 4/29/2011 13:59'!
removeSelector: selector
	"Removes a method named as selector"

	methods removeKey: selector ifAbsent:[]! !

!RGBehaviorDefinition methodsFor: 'subclassing' stamp: 'VeronicaUquillas 4/29/2011 09:12'!
removeSubclass: aRGBehaviorDefinition 
	"Removes aRGAbstractClassDefinition from the direct subclasses - without failing if does not exist"
	
	self subclasses remove: aRGBehaviorDefinition ifAbsent: []! !

!RGBehaviorDefinition methodsFor: 'accessing methods' stamp: 'VeronicaUquillas 3/21/2011 17:23'!
selectors
	"Retrieves the method selectors"

	^methods keys! !

!RGBehaviorDefinition methodsFor: 'compatibility' stamp: 'VeronicaUquillas 4/14/2011 11:13'!
soleInstance
	"to be depracated in the future"
	
	^self theNonMetaClass! !

!RGBehaviorDefinition methodsFor: 'printing' stamp: 'VeronicaUquillas 2/25/2011 19:25'!
storeOn: aStream 
	aStream nextPutAll: self name! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
subclasses
	"Retrieves the direct subclasses of the receiver.
	This value is kept as an annotation"

	^self annotationNamed: self class subclassesKey
		ifAbsentPut:[ OrderedCollection new ]! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
subclasses: aCollection
	"Stores direct subclasses of the receiver as an annotation"

	self annotationNamed: self class subclassesKey
		ifAbsentPut:[ aCollection ]! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/17/2010 11:01'!
superclass

	^superclass! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/19/2011 13:39'!
superclass: aRGBehaviorDefinition
	"The superclass is assigned.
	If aRGBehaviorDefinition is not nil the receiver is added as a subclass and the superclass assignment also happens for theMetaClass"

	superclass := aRGBehaviorDefinition.
	superclass notNil
	ifTrue: [ 
		self superclassName: aRGBehaviorDefinition name.
		aRGBehaviorDefinition addSubclass: self.
		self hasMetaclass
			ifTrue: [ self theMetaClass superclass: aRGBehaviorDefinition theMetaClass ] ]! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
superclassName
	"Retrieves the name of the superclass if exists"

	^self annotationNamed: self class superclassNameKey! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:21'!
superclassName: aSymbol

	self annotationNamed: self class superclassNameKey 
		put: aSymbol! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: ''!
theMetaClass

	self subclassResponsibility! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: ''!
theNonMetaClass

	self subclassResponsibility! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:22'!
traitCompositionSource
	"Retrieves aString representing the used traits"

	^self annotationNamed: self class traitCompositionSourceKey
		ifAbsentPut:[  '{}'  ]! !

!RGBehaviorDefinition methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:23'!
traitCompositionSource: anString
	"Stores aString representing the traits used by the receiver "

	self annotationNamed: self class traitCompositionSourceKey
		put: anString ! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 18:03'!
traitNames
	"Assuming that traits in a composition can be identified by
	testing for the first character being an uppercase character
	(and thus not a special character such as {, # etc.)"
	| tokens |
	
	tokens := Scanner new scanTokens: self traitCompositionSource.
	^tokens select: [:each | each first isUppercase].! !

!RGBehaviorDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/19/2011 14:01'!
traits
	"Retrieves ring traits based on the names in the traitComposition and from the environment if it is a ring object"
	
	^ self environment isRingObject 	
		ifTrue: [ self traitNames collect:[ :each| self environment traitNamed: each ] ]
		ifFalse:[ #() ]! !

!RGBehaviorDefinition methodsFor: 'accessing class hierarchy' stamp: 'VeronicaUquillas 3/19/2011 16:22'!
withAllSubclasses
	"if allSubclasses is stored should not affect the collection"

	^self allSubclasses, {self}! !

!RGBehaviorDefinition methodsFor: 'accessing class hierarchy' stamp: 'VeronicaUquillas 3/19/2011 16:22'!
withAllSuperclasses
	"if allSuperclasses is stored should not affect the collection"

	^self allSuperclasses, {self}! !

RGBehaviorDefinition subclass: #RGClassDescriptionDefinition
	instanceVariableNames: 'instanceVariables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGClassDescriptionDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:00' prior: 0!
RGClassDescriptionDefinition is the common parent for classes and metaclasses defining instance variables!

RGClassDescriptionDefinition subclass: #RGClassDefinition
	instanceVariableNames: 'metaClass comment classVariables poolDictionaries category package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGClassDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:01' prior: 0!
RGClassDefinition is the concrete representation of a class (no trait)!

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 5/15/2011 18:07'!
addClassVarNamed: aString
	| var |
	var:= self factory createClassVariableNamed: aString parent: self.
	self addVariable: var in: classVariables.
	^var! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 4/28/2011 21:27'!
addClassVariable: aRCClassVariable

	self addVariable: (aRCClassVariable parent: self) 
		in: classVariables! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 11/24/2010 17:24'!
addClassVariables: aCollection
	
	aCollection do: [:var | self addClassVarNamed: var ]! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 5/15/2011 18:07'!
addPoolDictNamed: poolName
	| var |
	var:= self factory createPoolVariableNamed: poolName parent: self.
	self addVariable: var in: poolDictionaries.
	^var! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 4/14/2011 11:19'!
addPoolDictionaries: aCollection
	
	aCollection do:[ :var| self addPoolDictNamed: var ]! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 9/29/2011 12:41'!
addPoolDictionary: aRGPoolVariableDefinition

	self addVariable: (aRGPoolVariableDefinition parent: self) 
		in: poolDictionaries! !

!RGClassDefinition methodsFor: 'managing pool users' stamp: 'VeronicaUquillas 8/2/2011 13:54'!
addUser: aRGClassDefinition
	"The receiver registers the aRGClassDefinition as an user.  An reinforces its status as a shared pool."
	
	aRGClassDefinition isClass ifFalse:[ ^self ].
	(aRGClassDefinition theNonMetaClass poolDictNamed: self name) isNil
		ifTrue: [ aRGClassDefinition theNonMetaClass addPoolDictNamed: self name ].
	self isPool: true.
	self users add: aRGClassDefinition theNonMetaClass! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 2/25/2011 20:38'!
allClassVarNames

	^self allClassVariables collect:[ :cvar| cvar name ]! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 4/29/2011 09:25'!
allClassVariables
	"Answer a collection of the receiver's classVariables, including those defined its superclasses"

	^self hasSuperclass 
		ifFalse:[ classVariables ]
		ifTrue:[ self superclass allClassVariables, classVariables ]! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 2/26/2011 13:48'!
allPoolDictNames

	^self allPoolDictionaries collect:[ :pool| pool name ]! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 4/29/2011 09:25'!
allPoolDictionaries
	"Answer a collection of the pools the receiver shares, including those defined  
	in the superclasses of the receiver."

	^self hasSuperclass 
		ifFalse:[ poolDictionaries ]
		ifTrue:[ self superclass allPoolDictionaries, poolDictionaries ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 11:15'!
category
	"retrieves a tag for its package"
	
	^category! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 3/21/2011 16:53'!
category: aSymbol
	"stores a tag for its package"

	category := aSymbol! !

!RGClassDefinition methodsFor: 'to be removed - compatibility for now' stamp: 'StephaneDucasse 7/26/2011 14:04'!
classSymbol

	^ self className! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 4/29/2011 09:39'!
classVarNamed: aString

	^classVariables detect:[ :v| v name = aString asSymbol ] ifNone:[ nil ]! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 4/29/2011 09:25'!
classVarNames
	"Answer a collection of the names of the class variables defined in the receiver."
	
	^classVariables collect:[ :cvar| cvar name ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/28/2011 21:30'!
classVariables

	^classVariables! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/2/2011 16:26'!
classVariables: aCollection

	classVariables:= aCollection! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/27/2011 10:45'!
comment
	"Retrieves the comment definition object"

	^comment! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/17/2011 13:46'!
comment: anObject
	"Sets a RGCommentDefinition object based on the argument"
	
	comment:=  self factory createComment:  anObject parent: self.! !

!RGClassDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 5/12/2011 14:41'!
hasComment

	^comment isEmptyOrNil not! !

!RGClassDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 12/2/2010 18:26'!
hasMetaclass

	^metaClass notNil! !

!RGClassDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 5/12/2011 10:09'!
hasStamp

	^self stamp isEmptyOrNil not! !

!RGClassDefinition methodsFor: 'managing pool users' stamp: 'VeronicaUquillas 8/2/2011 13:46'!
includesUser: aRGBehaviorDefinition

	^self users includes: aRGBehaviorDefinition! !

!RGClassDefinition methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 4/28/2011 21:30'!
initialize

	super initialize.
	classVariables:= OrderedCollection new.
	poolDictionaries:= OrderedCollection new.! !

!RGClassDefinition methodsFor: 'managing pool users' stamp: 'VeronicaUquillas 8/2/2011 14:03'!
isPool
	"The receiver is a shared pool if it inherits from SharedPool"
	
	^self annotationNamed: self class isPoolKey 
		ifAbsent: [ self superclassName == #SharedPool ]! !

!RGClassDefinition methodsFor: 'managing pool users' stamp: 'VeronicaUquillas 8/2/2011 13:41'!
isPool: aBoolean
	
	^self annotationNamed: self class isPoolKey put: aBoolean! !

!RGClassDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 15:01'!
isSameRevisionAs: aRGClassDefinition
	"This method look for equality of the properties of the receiver"

	^(super isSameRevisionAs: aRGClassDefinition)
		and:[ self superclassName == aRGClassDefinition superclassName
		and:[ self category = aRGClassDefinition category 
		and:[ (self classVarNames sorted: nil) = (aRGClassDefinition classVarNames sorted: nil) 
		and:[ (self poolDictNames sorted: nil) = (aRGClassDefinition poolDictNames sorted: nil)  
		and:[ self traitCompositionSource = aRGClassDefinition traitCompositionSource 
		and:[ ((self hasComment and:[ self comment isSameRevisionAs: aRGClassDefinition comment ]) or:[ self hasComment not ])
		and:[ (self theMetaClass isSameRevisionAs: aRGClassDefinition theMetaClass) ] ] ] ] ] ] ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 12:40'!
package
	"Retrieves the package in which this class is contained, if exists"
	
	^package! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/9/2011 14:28'!
package: aRGPackage
	"Sets the package in which this class is contained"
	
	package:= aRGPackage! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 4/29/2011 09:25'!
poolDictNamed: poolName

	^poolDictionaries detect:[ :v| v name = poolName asSymbol ] ifNone:[ nil ]! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 4/29/2011 09:25'!
poolDictNames
	
	^poolDictionaries collect:[ :pool| pool name ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/2/2011 13:37'!
poolDictionaries
	"Keeps the pool variable relationship of the receiver"

	^poolDictionaries! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/2/2011 16:26'!
poolDictionaries: aCollection

	poolDictionaries:= aCollection! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 4/29/2011 13:39'!
removeClassVarNamed: aString

	self removeVariable: (self classVarNamed: aString) from: classVariables! !

!RGClassDefinition methodsFor: 'class variables' stamp: 'VeronicaUquillas 4/29/2011 13:39'!
removeClassVariable: aRGClassVariableDefinition

	self removeVariable: aRGClassVariableDefinition from: classVariables! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 4/29/2011 13:40'!
removePoolDictNamed: poolName

	self removeVariable: (self poolDictNamed: poolName) from: poolDictionaries! !

!RGClassDefinition methodsFor: 'pool dictionaries' stamp: 'VeronicaUquillas 4/29/2011 13:39'!
removePoolDictionary: aRGPoolVariableDefinition

	self removeVariable: aRGPoolVariableDefinition from: poolDictionaries! !

!RGClassDefinition methodsFor: 'managing pool users' stamp: 'VeronicaUquillas 8/2/2011 13:46'!
removeUser: aRGClassDefinition
	"Removes this RGClassDefinition from the users of the receiver"
	
	aRGClassDefinition isClass ifFalse:[ ^self ].
	aRGClassDefinition theNonMetaClass removePoolDictNamed: self name.
	self users remove: aRGClassDefinition theNonMetaClass ifAbsent:[ ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/12/2011 10:08'!
stamp

	^self hasComment ifTrue:[ self comment stamp ] ifFalse:[ nil ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/17/2011 14:47'!
stamp: aString

	self hasComment
	ifTrue: [ self comment stamp: aString ]
	ifFalse:[ self factory createComment 
						 parent: self; 
						 stamp: aString ]! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 12/2/2010 18:27'!
theMetaClass
	
	^metaClass! !

!RGClassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 12/2/2010 18:27'!
theNonMetaClass

	^self! !

!RGClassDefinition methodsFor: 'managing pool users' stamp: 'VeronicaUquillas 8/2/2011 14:07'!
users
	"If the reciever is a SharedPool then retrieves its users"
	
	^self isPool
		ifTrue: [ self annotationNamed: self class usersKey ifAbsentPut: [ OrderedCollection new ] ]
		ifFalse:[ #() ]! !

!RGClassDefinition methodsFor: 'behavior' stamp: 'VeronicaUquillas 5/11/2011 11:36'!
withMetaclass
	"Registers explicitly the metaclass of a class"

	metaClass:= self factory createMetaclassOf: self.
! !

!RGClassDefinition methodsFor: 'behavior' stamp: 'VeronicaUquillas 5/12/2011 13:20'!
withMetaclass: aRGMetaclassDefinition
	"Registers explicitly the metaclass of a class"

	metaClass:= aRGMetaclassDefinition.
	metaClass baseClass: self.  
! !

!RGClassDescriptionDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 5/11/2011 14:03'!
= aRGClassDefinition

	^self class = aRGClassDefinition class 
		and: [ self name = aRGClassDefinition name 
			and: [ self environment = aRGClassDefinition environment ] ]! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 8/16/2011 15:24'!
addInstVarNamed: aString
	| var |
	var:= self factory createInstanceVariableNamed: aString parent: self.
	self addVariable: var in: instanceVariables.
	^var! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 4/28/2011 21:30'!
addInstanceVariable: aRGVariableDefinition
	"aRGVariableDefinition is a instance variable or class instance variable"

	self addVariable: (aRGVariableDefinition parent: self)
		in: instanceVariables! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 2/19/2011 00:42'!
addInstanceVariables: aCollection
	
	aCollection do: [:var | self addInstVarNamed: var ]! !

!RGClassDescriptionDefinition methodsFor: 'private' stamp: 'VeronicaUquillas 4/29/2011 09:23'!
addVariable: aRGVariableDefinition in: aCollection
	"Adds a RGVariableDefinition in the collection received" 

	aCollection add: aRGVariableDefinition! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 2/25/2011 20:38'!
allInstVarNames

	^self allInstanceVariables collect:[ :ivar| ivar name ]! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 4/29/2011 09:24'!
allInstanceVariables
	"Answer a collection of the receiver's instanceVariables, including those defined in its superclass"

	^self hasSuperclass 
		ifFalse:[ instanceVariables ]
		ifTrue:[ self superclass allInstanceVariables, instanceVariables ]! !

!RGClassDescriptionDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 2/25/2011 20:17'!
hash

	^self name hash bitXor: self class hash! !

!RGClassDescriptionDefinition methodsFor: 'initialize-release' stamp: 'VeronicaUquillas 11/24/2010 14:00'!
initialize

	super initialize.
	instanceVariables:= OrderedCollection new.! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 4/29/2011 09:24'!
instVarNamed: aString

	^instanceVariables detect:[ :v| v name = aString asSymbol ] ifNone:[ nil ]! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 4/29/2011 09:24'!
instVarNames
	"Answer a collection of the names of the instance variables defined in the receiver."
	
	^instanceVariables collect:[ :ivar| ivar name ]! !

!RGClassDescriptionDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/16/2010 11:20'!
instanceVariables

	^instanceVariables! !

!RGClassDescriptionDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 2/26/2011 12:02'!
instanceVariables: aCollection

	instanceVariables:= aCollection! !

!RGClassDescriptionDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/19/2010 11:59'!
isClass

	^true! !

!RGClassDescriptionDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 14:31'!
isSameRevisionAs: aRGClassDescriptionDefinition
	"This method look for equality of the properties of the receiver"
	"Instances variables are compared at the level of names but without any sorting"

	^(super isSameRevisionAs: aRGClassDescriptionDefinition)
		and:[ (self instVarNames sorted: nil) = (aRGClassDescriptionDefinition instVarNames sorted: nil) ]! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 4/29/2011 13:38'!
removeInstVarNamed: aString

	self removeVariable: (self instVarNamed: aString) from: instanceVariables! !

!RGClassDescriptionDefinition methodsFor: 'instance variables' stamp: 'VeronicaUquillas 4/29/2011 13:38'!
removeInstanceVariable: aRGVariableDefinition
	"aRGVariableDefinition is a instance variable or class instance variable"

	self removeVariable: aRGVariableDefinition from: instanceVariables! !

!RGClassDescriptionDefinition methodsFor: 'private' stamp: 'VeronicaUquillas 4/29/2011 13:37'!
removeVariable: aRGVariableDefinition from: aCollection
	"Removes a variable from a particular collection.
	This behavior is the same for any kind of variable"
	aCollection remove: aRGVariableDefinition ifAbsent:[]! !

RGClassDescriptionDefinition subclass: #RGMetaclassDefinition
	instanceVariableNames: 'baseClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGMetaclassDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:05' prior: 0!
RGMetaclassDefinition is a concrete representation of metaclasses of classes (traits are excluded)!

!RGMetaclassDefinition class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 2/15/2011 22:05'!
class: anORClassDefinition

	^self new
		baseClass: anORClassDefinition;
		yourself! !

!RGMetaclassDefinition methodsFor: 'variables' stamp: 'VeronicaUquillas 2/26/2011 13:51'!
allClassVarNames

	^self theNonMetaClass allClassVarNames! !

!RGMetaclassDefinition methodsFor: 'variables' stamp: 'VeronicaUquillas 2/26/2011 13:51'!
allPoolDictNames

	^self theNonMetaClass allPoolDictNames! !

!RGMetaclassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/20/2011 17:34'!
baseClass: aRGClassDefinition

	baseClass:= aRGClassDefinition.
	self name: (baseClass name, ' class') asSymbol! !

!RGMetaclassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/11/2011 11:38'!
environment

	^baseClass environment! !

!RGMetaclassDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/17/2010 15:52'!
isMeta

	^true! !

!RGMetaclassDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 7/18/2011 14:32'!
isSameRevisionAs: aRGMetaclassDefinition
	"This method look for equality of the properties of the receiver"

	^(super isSameRevisionAs: aRGMetaclassDefinition)
		and:[ self traitCompositionSource = aRGMetaclassDefinition traitCompositionSource ]! !

!RGMetaclassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 3/3/2011 10:03'!
package

	^baseClass package! !

!RGMetaclassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/16/2011 17:07'!
realClass

	^baseClass realClass theMetaClass! !

!RGMetaclassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/17/2010 14:43'!
theMetaClass

	^self! !

!RGMetaclassDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/17/2010 14:42'!
theNonMetaClass

	^baseClass! !

RGBehaviorDefinition subclass: #RGTraitDescriptionDefinition
	instanceVariableNames: 'users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGTraitDescriptionDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:02' prior: 0!
RGTraitDescriptionDefinition is the common parent for traits and metatraits that know their users (ie. classes/metaclasses)!

RGTraitDescriptionDefinition subclass: #RGMetatraitDefinition
	instanceVariableNames: 'baseClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGMetatraitDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:04' prior: 0!
RGMetatraitDefinition is a concrete representation of metaclasses of traits!

!RGMetatraitDefinition class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 2/15/2011 23:15'!
class: anORTraitDefinition

	^self new
		baseClass: anORTraitDefinition;
		yourself! !

!RGMetatraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/20/2011 17:35'!
baseClass: anRGTraitDefinition

	baseClass:= anRGTraitDefinition.
	self name: (baseClass name, ' classTrait') asSymbol! !

!RGMetatraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/11/2011 11:38'!
environment

	^baseClass environment! !

!RGMetatraitDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/17/2010 15:52'!
isMeta

	^true! !

!RGMetatraitDefinition methodsFor: 'testing' stamp: 'StephaneDucasse 7/19/2011 10:26'!
isSameRevisionAs: aRGMetatraitDefinition
	"This method look for equality of the properties of the receiver"

	^(super isSameRevisionAs: aRGMetatraitDefinition)
		and: [ self traitCompositionSource = aRGMetatraitDefinition traitCompositionSource ]! !

!RGMetatraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 3/3/2011 10:03'!
package

	^baseClass package! !

!RGMetatraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 9/16/2011 17:07'!
realClass

	^baseClass realClass theMetaClass! !

!RGMetatraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/24/2010 22:32'!
theMetaClass

	^self! !

!RGMetatraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 12/2/2010 18:09'!
theNonMetaClass

	^baseClass! !

RGTraitDescriptionDefinition subclass: #RGTraitDefinition
	instanceVariableNames: 'metaClass comment category package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGTraitDefinition commentStamp: 'VeronicaUquillas 4/19/2011 16:03' prior: 0!
RGTraitDefinition is the concrete representation of a trait (no class)!

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/13/2011 21:15'!
category
	"Retrieves the tag-package associated to the receiver"

	^category! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/24/2010 13:11'!
category: aSymbol

	category:= aSymbol! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 17:32'!
classVariables

	^#()! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/9/2011 14:27'!
comment
	"Retrieves the comment definition object associated to the receiver"
	
	^comment! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/17/2011 13:46'!
comment: anObject
	"Sets a RGCommentDefinition object based on the argument"
	
	comment:= self factory createComment:  anObject parent: self! !

!RGTraitDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 5/12/2011 14:45'!
hasComment

	^comment isEmptyOrNil not! !

!RGTraitDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 12/2/2010 18:08'!
hasMetaclass

	^metaClass notNil! !

!RGTraitDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 5/12/2011 10:11'!
hasStamp

	^self stamp isEmptyOrNil not! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 17:33'!
instanceVariables

	^#()! !

!RGTraitDefinition methodsFor: 'testing' stamp: 'StephaneDucasse 7/19/2011 10:25'!
isSameRevisionAs: aRGTraitDefinition
	"This method look for equality of the properties of the receiver"

	^(super isSameRevisionAs: aRGTraitDefinition)
		and: [ self category = aRGTraitDefinition category 
		and: [ self traitCompositionSource = aRGTraitDefinition traitCompositionSource 
		and: [ ((self hasComment and: [ self comment isSameRevisionAs: aRGTraitDefinition comment ]) or:[ self hasComment not ])
		and: [ (self theMetaClass isSameRevisionAs: aRGTraitDefinition theMetaClass) ] ] ] ]! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/13/2011 21:16'!
package
	"Retrieves the package in which this trait is contained"
	
	^package! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/20/2011 17:36'!
package: aRGPackage
	"Sets the package in which this trait is contained"
	
	package:= aRGPackage! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 17:32'!
poolDictionaries

	^#()! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/12/2011 10:09'!
stamp

	^self hasComment ifTrue:[ self comment stamp ] ifFalse:[ nil ]! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 5/17/2011 14:48'!
stamp: aString

	self hasComment
	ifTrue: [ self comment stamp: aString ]
	ifFalse:[ self factory createComment 
						 parent: self; 
						 stamp: aString ]! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 12/2/2010 18:08'!
theMetaClass

	^metaClass! !

!RGTraitDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 11/24/2010 22:32'!
theNonMetaClass

	^self! !

!RGTraitDefinition methodsFor: 'behavior' stamp: 'VeronicaUquillas 5/11/2011 11:36'!
withMetaclass
	"Registers explicitly the metaclass of a trait"

	metaClass:= self factory createMetatraitOf: self.
! !

!RGTraitDefinition methodsFor: 'behavior' stamp: 'VeronicaUquillas 5/12/2011 13:20'!
withMetaclass: aRGMetatraitDefinition
	"Registers explicitly the metaclass of a class"

	metaClass:= aRGMetatraitDefinition.
	metaClass baseClass: self.  
! !

!RGTraitDescriptionDefinition methodsFor: 'adding/removing users' stamp: 'VeronicaUquillas 7/27/2011 18:15'!
addUser: aRGBehavior

	self users add: aRGBehavior! !

!RGTraitDescriptionDefinition methodsFor: 'variables' stamp: 'VeronicaUquillas 4/14/2011 11:28'!
allClassVarNames

	^#()! !

!RGTraitDescriptionDefinition methodsFor: 'variables' stamp: 'VeronicaUquillas 4/12/2011 10:04'!
allInstVarNames

	^#()! !

!RGTraitDescriptionDefinition methodsFor: 'variables' stamp: 'VeronicaUquillas 4/14/2011 11:28'!
allPoolDictNames

	^#()! !

!RGTraitDescriptionDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 11/19/2010 11:59'!
isTrait

	^true! !

!RGTraitDescriptionDefinition methodsFor: 'adding/removing users' stamp: 'VeronicaUquillas 7/27/2011 18:15'!
removeUser: aRGBehavior

	self users remove: aRGBehavior ifAbsent:[]! !

!RGTraitDescriptionDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/27/2011 18:10'!
users
	"Retrieves the receiver's classes and traits"
	
	^users ifNil:[ users := OrderedCollection new ]! !

!RGTraitDescriptionDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 2/16/2011 00:03'!
users: aCollection

	users:= aCollection! !

!RGGlobalDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/2/2011 13:32'!
= aRGGlobalDefinition
	"This method look for equality of the properties of the receiver"
	"Verifies the class and the name of the receiver"

	^self class = aRGGlobalDefinition class
		and:[ self name == aRGGlobalDefinition name ]! !

!RGGlobalDefinition methodsFor: 'comparing' stamp: 'VeronicaUquillas 8/2/2011 13:32'!
hash
	"Hash is re-implemented because #= is re-implemented"
	
	^self class hash bitXor: (self name hash)! !

!RGGlobalDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/15/2011 14:58'!
parent
	"Retrieves the namespace in which the receiver is known"
	
	^self environment! !

RGGlobalDefinition subclass: #RGGlobalVariableDefinition
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ring-Core-Kernel'!

!RGGlobalVariableDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 5/9/2011 15:33'!
addInContainer: aRGContainer

	aRGContainer addGlobalVariable: self! !

!RGGlobalVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/13/2011 16:31'!
isGlobalVariable

	^true! !

!RGGlobalVariableDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 5/9/2011 15:33'!
isIncludedInContainer: aRGContainer

	^aRGContainer includesGlobalVariable: self! !

!RGGlobalVariableDefinition methodsFor: 'testing' stamp: 'VeronicaUquillas 4/14/2011 10:03'!
isVariable

	^true! !

!RGGlobalVariableDefinition methodsFor: 'managing container' stamp: 'VeronicaUquillas 5/9/2011 15:33'!
removeFromContainer: aRGContainer

	aRGContainer removeGlobalVariable: self! !

!RGGlobalVariableDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 8/2/2011 13:32'!
value
	"Retrieves the current value of the global variable"

	^ value! !

!RGGlobalVariableDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 7/29/2011 09:46'!
value: anObject

	value := anObject! !

!RGNamedDefinition class methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:16'!
authorAliasKey

	^#authorAlias! !

!RGNamedDefinition class methodsFor: 'annotations' stamp: 'VeronicaUquillas 7/28/2011 10:11'!
fullNameKey

	^#fullName! !

!RGNamedDefinition class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 4/14/2011 10:38'!
named: aSymbol

	^self new
		name: aSymbol;
		yourself! !

!RGNamedDefinition class methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:16'!
timeStampKey

	^#timestamp! !

!RGNamedDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 14:22'!
fullName

	^self name! !

!RGNamedDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 10:38'!
name

	^name! !

!RGNamedDefinition methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/14/2011 10:38'!
name: anObject
	"a symbol or a string should be expected"
	
	name:= anObject! !

!RGObject class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 4/28/2011 17:29'!
factory
	"Retrives the factory in charge of object creation"

	^RGFactory current! !

!RGObject class methodsFor: 'class initialization' stamp: 'VeronicaUquillas 4/14/2011 15:05'!
onEnvironment: anObject
	"no used yet"

	^self new
		environment: anObject;
		yourself! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 11/25/2010 17:25'!
annotationNamed: annotationName 
	"Answer the value of the annotation named <annotationName>, or 
	nil if the annotation is not defined for the receiver."
	
	^ self
		annotationNamed: annotationName
		ifAbsent: [ nil ]! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 11/25/2010 17:25'!
annotationNamed: annotationName ifAbsent: exceptionBlock 
	"Answer the value of the annotation named <annotationName>. If the 
	annotation is not defined for the receiver, answer the result of 
	evaluating the <exceptionBlock>."

	self hasAnnotations
		ifFalse: [ ^exceptionBlock value ].

	^self annotations
			at: annotationName
			ifAbsent: [ exceptionBlock value ]! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 3/22/2011 16:41'!
annotationNamed: annotationName ifAbsentPut: blockValue 

	^self annotations
		at: annotationName
		ifAbsentPut: blockValue! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 7/19/2011 14:21'!
annotationNamed: annotationName put: value 

	self annotations
		at: annotationName
		put: value! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 11/25/2010 17:25'!
annotationNames
	
	self hasAnnotations
		ifFalse:[ ^OrderedCollection new ].
	^annotations keys! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 6/28/2011 15:16'!
annotations

	^annotations ifNil:[ annotations:= IdentityDictionary new ]! !

!RGObject methodsFor: 'accessing' stamp: 'VeronicaUquillas 4/28/2011 17:29'!
factory
	"Retrives the factory in charge of object creation"

	^self class factory! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 11/25/2010 17:25'!
hasAnnotationNamed: annotationName
	"Answer <true> if the receiver contains the annotation named <annotationName>. "
	
	self hasAnnotations	
		ifFalse:[ ^false ].
	^annotations includesKey: annotationName! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 11/25/2010 17:25'!
hasAnnotations

	^annotations notNil! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 5/17/2011 12:50'!
isAccess 

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 5/17/2011 15:07'!
isAssociation

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 3/20/2011 16:55'!
isClass

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 4/27/2011 10:40'!
isComment

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 4/19/2011 15:57'!
isGlobalVariable

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 5/17/2011 12:49'!
isInheritance

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 5/17/2011 12:49'!
isInvocation

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 3/20/2011 16:55'!
isMethod

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 4/19/2011 15:57'!
isNamespace

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 5/12/2011 13:42'!
isOrganization 

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 3/20/2011 16:54'!
isPackage

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 4/19/2011 15:57'!
isPool

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 5/17/2011 12:50'!
isReference 

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 4/14/2011 16:32'!
isRingObject

	^true! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 3/20/2011 17:56'!
isSlice

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 3/20/2011 16:55'!
isTrait

	^false! !

!RGObject methodsFor: 'testing types' stamp: 'VeronicaUquillas 3/20/2011 16:54'!
isVariable

	^false! !

!RGObject methodsFor: 'annotations' stamp: 'VeronicaUquillas 11/25/2010 17:26'!
removeAnnotationNamed: annotationName 
	"Remove the annotation named <annotationName>. Fails quietly if there 
	is no annotation with the given name."
	
	(self hasAnnotationNamed: annotationName)
		ifTrue:[ annotations removeKey: annotationName ]! !
RGFactory initialize!
